// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

#include "SnoopyGlobals.h"
#include "FeeGlobal.h"
#include "BaseballSprite.h"
#include "FeeManager.h"
#include "FeeSound.h"
#include "sge_misc.h"
#include "PlayerSprite.h"
#include "GameOptions.h"
#include <math.h> //brauch ich wohl leider...

const int BASEBALL_SPEED_Y = -12;

BaseballSprite::BaseballSprite(int x, int y, FeeManager *derManager):MovingSprite(x,y, derManager)
{
	myKind=BASEBALL_KIND;
	name="Charlies Baseball";
	frame = 0;
	hasHitWall = false;
}

BaseballSprite::~BaseballSprite()
{
}

void BaseballSprite::Update(float time)
{
	if (!face)
		InitFace();
	if (!hasHitWall)	// War der Ball schon an der Wand?
	{
		speedX=-18;
		speedY=BASEBALL_SPEED_Y;

		if (x <= 0)
		{
			hasHitWall = true;
			ComputeSpeed(); // (Nicht die MovingSprite-Standardfunktion)
		}
	}
	else
	{
		if (DoAction(time))
			speedY++;
	}

	MovingSprite::Update(time);

	if ((x > myManager->maxX) || (y > myManager->maxY))
		toBeRemoved = true;
}

void BaseballSprite::Hit(FeeSprite *hitBy)
{
	// Immer weg, auer hitBy ist ein SkySprite
	if (hitBy->myKind != SKY_KIND) toBeRemoved = true;
}


void BaseballSprite::InitFace()
{
	InitFaceFromIMG(myManager->options->GetDataPath("Grafiken/Baseball.tga"));
}

void BaseballSprite::ComputeSpeed()
{
	// Spieler aussuchen
	// Zuerst die beiden Spielersprites raussuchen:
	PlayerSprite *dieSpieler[2];
	int gesuchterSpieler = 0;
	while (gesuchterSpieler < 2)
	{
		dieSpieler[gesuchterSpieler++] = dynamic_cast<PlayerSprite*>(myManager->GetNextSprite(PLAYER_KIND));
	}
	// Jetzt einen waehlen:
	gesuchterSpieler = sge_Random(0,1);
	speedX = 18;
	speedY = BASEBALL_SPEED_Y;
	for (int i = 0; i < 2; i++)
	{
		// Hoffentlich will niemand wissen, wie oft ich mich hier verrechnet habe. Ich hab Angst vor dem Mathe-ABI!!!
		gesuchterSpieler = (gesuchterSpieler == 1) ? 0 : 1;
		if (dieSpieler[gesuchterSpieler]->GetDead()) continue; // Spieler ist tot, nicht auf ihn zielen
		double dieXPos = dieSpieler[gesuchterSpieler]->x;
		double dieYPos = dieSpieler[gesuchterSpieler]->y;
		double derXSpeed = dieSpieler[gesuchterSpieler]->speedX;
		double derYSpeed = dieSpieler[gesuchterSpieler]->speedY;
		double zwischenergebnis = exp(log(abs(-speedY+derYSpeed+1/2))*2)-2*y+2*dieYPos; 
		if (zwischenergebnis < 0) continue;	// Glueck fuer den Spieler, er ist zu weit weg. Dann eben der andere.
		zwischenergebnis = - speedY + derYSpeed + 1/2 + sqrt(zwischenergebnis);
		if (zwischenergebnis < 0) continue;	// Das wird nichts. Spieler wechseln.
		speedX = derXSpeed - (x - dieXPos)/zwischenergebnis;
		if ((speedX < 3) || (speedX > 25))	// Zu unrealistisch. Anderen Spieler versuchen bzw. irgendwohinbratzen.
			if (!i) continue; else speedX = 18;
		 else
			break;	// Alles OK. Jetzt ist einer wohl dran.
	}
}