// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

#include "SnoopyGlobals.h"
#include "FeeGlobal.h"
#include "CharlieSprite.h"
#include "ShotSprite.h"
#include "FeeManager.h"
#include "FeeSound.h"
#include "FeeImageManager.h"
#include "BaseballSprite.h"
#include "sge_misc.h"
#include "FeeHelper.h"
#include "GameOptions.h"

const int CHARLIE_FRAME_CHANGE = 2;

CharlieSprite::CharlieSprite(int x, FeeManager *derManager):FeeSprite(x,derManager->maxY-62, derManager, false)
{
	myKind=CHARLIE_KIND;
	name="Charlie Brown";
	frame = 9;
	changeFrame = CHARLIE_FRAME_CHANGE;
	isHit = 0;
	myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/CharlieBrown.tga"),10);
	myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/CharlieBrownTreffer.tga"),5);
	InitFace();
}

CharlieSprite::~CharlieSprite()
{
}

void CharlieSprite::Update(float time)
{
	if (!face)
	{
		InitFace();
	}
	if (DoAction(time))
	{
		if (isHit)
		{
			changeFrame--;
			if (!changeFrame)
			{
				frame++;
				isHit++;
				if (frame == 5) frame = 3;	// Sternchen sehen
				changeFrame = CHARLIE_FRAME_CHANGE;
				if (isHit == 50) // Genug "ausgeruht"
				{
					isHit = 0;
					frame = 9;
					x += 65;
					y += 22;
				}
				InitFace();
			}
		}
		if (!isHit)
		{
			if ((frame != 8) || (!myManager->GetSprite(BASEBALL_KIND)))  // Solange, wie der Ball fliegt, in die Luft sehen
				if ((frame != 9) || (sge_Random(25, 50) == 42))  // Nur ab und zu werfen
					changeFrame--;
			if (!changeFrame)
			{
				frame++;
				frame %= 10;
				changeFrame = CHARLIE_FRAME_CHANGE;
				if (frame == 4) // Wurf
				{
					new BaseballSprite(x+2, y+15, myManager);
				}
				InitFace();
			}
		}
	}
}

void CharlieSprite::Hit(FeeSprite *hitBy)
{
	if (isHit) return;
	switch (hitBy->myKind)
	{
		case MG_KIND:
		case ARROW_KIND:
		case BASEBALL_KIND:
		case EXPLO_KIND:
		{
			theCharlieHitSound->Play();
			isHit = 1;			
			frame = 0;
			x -= 65;
			y -= 22;
			InitFace();
		}
		default:
		break;
	}
}

void CharlieSprite::InitFace()
{
	if (isHit)
	{
		face = myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/CharlieBrownTreffer.tga"),frame);
		collisionMask = myManager->imageManager->GetFrameMask(myManager->options->GetDataPath("Grafiken/CharlieBrownTreffer.tga"),frame);
	}
	else
	{
		face = myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/CharlieBrown.tga"),frame);
		collisionMask = myManager->imageManager->GetFrameMask(myManager->options->GetDataPath("Grafiken/CharlieBrown.tga"),frame);
	}
}

