// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// FeeImageList.cpp: implementation of the FeeImageList class.
//
//////////////////////////////////////////////////////////////////////

#include "CON_console.h"
#include "FeeImageList.h"
#include "FeeGlobal.h"
#include "SDL.h"
#include "SDL_image.h"
#include <string.h>
#include "sge_collision.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FeeImageList::FeeImageList(SDL_Surface *face,char *name)
{
	strcpy(this->name,name);
	myImage=face;
	next=NULL;
	myMask=NULL;
#ifdef HAVE_GL_H
	myTex=0;
#endif
}

FeeImageList::~FeeImageList()
{
	if (myImage) SDL_FreeSurface(myImage);
	if (myMask) sge_destroy_cmap(myMask);
#ifdef _DEBUG
	//CON_ConOut("Freed %s...",name); // _echt_ lahm
#endif
#ifdef HAVE_GL_H
	if (myTex) 
		glDeleteTextures_priv(1,&myTex);
#endif
}

bool FeeImageList::IsImage(char * filename)
{
	if (strcmp(filename,name))
		return false;
	else
	{
		return true;
	}
}

sge_cdata *FeeImageList::GetMask()
{
	if (!myMask)
	{
		SDL_LockSurface(myImage);
		myMask=sge_make_cmap(myImage);
		SDL_UnlockSurface(myImage);
	}
	return myMask;
}

SDL_Surface *FeeImageList::GetImage()
{
	return myImage;
}

#ifdef HAVE_GL_H
GLuint FeeImageList::GetTexture()
{
	if (myTex==0)
	{
		if (myImage)
		{			
			myTex=FaceToTexture(myImage);
#ifdef WITH_VOODOO_FIX
#if 0
			if ((myTex==0)&&((myImage->w>256)||(myImage->h>256))) // if the surface is larger than 256*256, stretch down and then convert it.
			{
				SDL_Surface *temp; // TODO: This sucks.
				temp = myImage;

				myImage = SDL_CreateRGBSurface(myImage->flags, 50, 50, myImage->format->BitsPerPixel, myImage->format->Rmask, myImage->format->Gmask, myImage->format->Bmask, myImage->format->Amask);

				SDL_Rect srcrect={0,0,temp->w,temp->h};
				SDL_Rect dstrect={0,0,myImage->w,myImage->h};
				SDL_SoftStretch(temp, &srcrect, myImage, &dstrect);

				SDL_FreeSurface(temp);

				myTex=FaceToTexture(myImage);
			}
#endif
#endif
		}
	}
	return myTex;
}

GLuint FaceToTexture(SDL_Surface *face)
{
	GLuint tex_id;
	GLuint format;
	GLuint x, y, w, h;
	GLuint bpp;	unsigned char *data;

	SDL_PixelFormat fmt;

	w=SafeSize(face->w);
	h=SafeSize(face->h);


	fmt.palette=NULL;
    fmt.BitsPerPixel=32;
    fmt.BytesPerPixel=4;
	fmt.Rmask=0xFF000000;
	fmt.Gmask=0x00FF0000;
	fmt.Bmask=0x0000FF00;
	fmt.Amask=0x000000FF;

	fmt.Rshift=48;
	fmt.Gshift=32;
	fmt.Bshift=16;
	fmt.Ashift=0;

	fmt.Rloss=0;
	fmt.Gloss=0;
	fmt.Bloss=0;
	fmt.Aloss=0;

	fmt.alpha=face->format->alpha;
	fmt.colorkey=face->format->colorkey;

	face=SDL_ConvertSurface(face,&fmt,0);

	SDL_LockSurface(face);

	data = (unsigned char*) calloc(w * h * 4, sizeof(char));

	// copy every pixel 
	for(y = 0; y < face->h; y++)
	{
		for(x = 0; x < face->w; x++) 
		{
			Uint32 bufp;
			bufp = sge_GetPixel(face, x, y);
			*(data + 4 * (y * w + x) + 0) = bufp >> face->format->Rshift;
			*(data + 4 * (y * w + x) + 1) = bufp >> face->format->Gshift;
			*(data + 4 * (y * w + x) + 2) = bufp >> face->format->Bshift;
			if(face->format->Amask!=0) 
				*(data + 4 * (y * w + x) + 3) = bufp >> face->format->Ashift;
			else if ((bufp==face->format->colorkey)&&(face->flags&SDL_SRCCOLORKEY))
			{
				*(data + 4 * (y * w + x) + 0)=0;
				*(data + 4 * (y * w + x) + 1)=0;
				*(data + 4 * (y * w + x) + 2)=0;
				*(data + 4 * (y * w + x) + 3)=SDL_ALPHA_TRANSPARENT;
			}
			else 
				*(data + 4 * (y * w + x) + 3)=face->format->alpha;
		}
	}

	SDL_UnlockSurface(face);


	format = GL_RGBA;
	
	// load the texture into OGL 
	glGenTextures_priv(1, &tex_id);
	glBindTexture_priv(GL_TEXTURE_2D, tex_id);
	glPixelStorei_priv(GL_UNPACK_ALIGNMENT, 1);
	glTexImage2D_priv(GL_TEXTURE_2D, 0, 4, w, h, 0, (GLenum)format, GL_UNSIGNED_BYTE, data);

	glTexEnvi_priv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri_priv(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri_priv(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf_priv(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf_priv(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	SDL_FreeSurface(face);

	free(data);
	
	return tex_id;
}


int SafeSize(int i)
{
	int p;

	for (p = 0; p < 24; p++)
		if (i <= (1<<p))
			return 1<<p;

	return 128;
}

void FeeImageList::UpdateTexture()
{
	glDeleteTextures_priv(1,&myTex);
	myTex=0;
}
#endif
