// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// FeeImageManager.cpp: implementation of the FeeImageManager class.
//
//////////////////////////////////////////////////////////////////////

#include "FeeImageList.h"
#include "FeeImageManager.h"
#include "CON_console.h"
#include "SDL_image.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FeeImageManager::FeeImageManager()
{
	default_image=SDL_CreateRGBSurfaceFrom((void*)achtung.pixel_data ,achtung.width, achtung.height, 32, achtung.width*achtung.bytes_per_pixel, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000);
	imageList=NULL;
}

FeeImageManager::~FeeImageManager()
{
	SDL_FreeSurface(default_image);
	FeeImageList *list=imageList;
	FeeImageList *temp=imageList;
	while (list)
	{
		temp=list;
		list = list->next;
		delete temp;
	}
}

SDL_Surface *FeeImageManager::GetImage(char *filename, int flags)
{
	FeeImageList *list;
	list=FindImage(filename);
	if (!list)
	{
		SDL_Surface *face;
		face=LoadSurface(filename,flags);
		if (face)
			list=AddImage(face,filename);
		else
			list=AddImage(default_image,filename);
	}
	return list->GetImage();
}

SDL_Surface *FeeImageManager::GetSimpleImage(char *filename)
{
	return GetImage(filename,IMAGE_SIMPLE);
}

SDL_Surface *FeeImageManager::GetAlphaSurface(char *filename)
{
	return GetImage(filename, IMAGE_CONVERT_ALPHA);
}

void FeeImageManager::PrecacheFrames(char *filename,int number)
{
	char *name=(char*)malloc(strlen(filename)+5);

	sprintf(name,"%s:%i",filename,0);
	if (FindImage(name))
	{
		free(name);
		return;						// gibt's ja schon
	}

	SDL_Surface *face=IMG_Load(filename);
	SDL_Rect rect={0,0,face->w/number,face->h};
	SDL_Surface *frame;
	SDL_Surface *temp;

	SDL_SetAlpha(face,0,0);

	for (int i=0;i<=number;i++)
	{
		temp=SDL_CreateRGBSurface(SDL_SRCCOLORKEY,face->w/number,face->h,32,0xFF000000,0x00FF0000,0x0000FF00,0x000000FF);
		
		if (face->format->palette)
		{
			frame=SDL_ConvertSurface(temp,face->format,SDL_SRCCOLORKEY);
			SDL_FreeSurface(temp);
			SDL_SetColorKey(frame, SDL_SRCCOLORKEY|SDL_RLEACCEL,*(Uint8 *)face->pixels);
			temp=frame;
		}


		rect.x=(face->w/number)*i;
		SDL_BlitSurface(face,&rect,temp,NULL);
		sprintf(name,"%s:%i",filename,i);
		if (face->format->palette)
			frame=SDL_DisplayFormat(temp);
		else
			frame=SDL_DisplayFormatAlpha(temp);

		SDL_FreeSurface(temp);

		AddImage(frame,name);
	}
	SDL_FreeSurface(face);
	free(name);
}

SDL_Surface *FeeImageManager::GetFrame(char *filename,int frame)
{
	char *name=(char*)malloc(strlen(filename)+5);
	sprintf(name,"%s:%i",filename,frame);
	FeeImageList *list=FindImage(name);
	free(name);
	if (list)
		return list->GetImage();
	else
		return default_image;
}

FeeImageList *FeeImageManager::AddImage(SDL_Surface *face,char *name)
{
	FeeImageList *list = imageList;
	imageList = new FeeImageList(face,name); // Wenn es keine Liste gibt, erstellen.
	imageList->next=list;
	return imageList;
}

FeeImageList *FeeImageManager::FindImage(char *name)
{
	if (!(imageList))
		return NULL;

	FeeImageList *list = imageList;
	while (list)
	{
		if (list->IsImage(name))
		{
			return list;
		}
		
		list=list->next;
	}
	return NULL;
}

void FeeImageManager::RemoveImage(char *name)
{
	if (!(imageList))
		return;

	FeeImageList *list = imageList;
	FeeImageList *previous = NULL;
	while (list)
	{
		if (list->IsImage(name))
		{
			if ((previous))
			{
				previous->next=list->next;
			}
			else
			{
				imageList=list->next;
			}
			delete list;
			break;
		}
		previous=list;
		list=list->next;
	}
}

SDL_Surface *FeeImageManager::LoadSurface(char *name, int flags)
{
	SDL_Surface *temp;
	SDL_Surface *face;
	face=NULL;

	temp=IMG_Load(name);
	if (temp)
	{
		if (flags&IMAGE_CONVERT_ALPHA)
		{
			face=SDL_DisplayFormatAlpha(temp);
			if (face!=NULL)
				SDL_FreeSurface(temp);
			else
			{
				face=temp;
				printf("Could not convert surface. (%s)",SDL_GetError());
			}

			ConvertToAlphaSurface(face);
			temp=face;
		}
		if (!(flags&IMAGE_SIMPLE))
		{
			if (temp->format->palette) 
			{
				SDL_SetColorKey(temp, (SDL_SRCCOLORKEY|SDL_RLEACCEL),
								*(Uint8 *)temp->pixels);
				face = SDL_DisplayFormat(temp);
			}
			else
				face = SDL_DisplayFormatAlpha(temp);
		}
		else
			face = SDL_DisplayFormat(temp);

		if (face!=NULL)
			SDL_FreeSurface(temp);
		else
		{
			face=temp;
			printf("Could not convert surface. (%s)",SDL_GetError());
		}

#ifdef _DEBUG
		CON_ConOut("Loaded %s...",name);
#endif
	}
	else
		CON_ConOut("Loading image %s failed (%s)",name,SDL_GetError());
	return face;
}

void FeeImageManager::ConvertToAlphaSurface(SDL_Surface *face)
{
	int32 i;
	SDL_LockSurface(face);
	int32 alpha=0;
	switch (face->format->BytesPerPixel)	// und neuen Alphawert setzen
	{
	case 1:
		for (i = 0; i < (face->w * face->h); i++)			
		{
			alpha = *((Uint8 *)face->pixels + i) & face->format->Rmask;
			alpha >>=face->format->Rshift;
			alpha <<=face->format->Ashift;

			*((Uint8 *)face->pixels + i) = 0;

			*((Uint8 *)face->pixels + i) |= face->format->Rmask;
			*((Uint8 *)face->pixels + i) |= face->format->Gmask;
			*((Uint8 *)face->pixels + i) |= face->format->Bmask;

			*((Uint8 *)face->pixels + i) |= alpha;
		}
		break;		
	case 2:
		for (i = 0; i < (face->w * face->h); i++)			
		{
			alpha = *((Uint16 *)face->pixels + i) & face->format->Rmask;
			alpha >>=face->format->Rshift;
			alpha <<=face->format->Ashift;

			*((Uint16 *)face->pixels + i) = 0;

			*((Uint16 *)face->pixels + i) |= face->format->Rmask;
			*((Uint16 *)face->pixels + i) |= face->format->Gmask;
			*((Uint16 *)face->pixels + i) |= face->format->Bmask;

			*((Uint16 *)face->pixels + i) |= alpha;
		}
		break;
		
	case 4:
		for (i = 0; i < (face->w * face->h); i++)			
		{
			alpha = *((Uint32 *)face->pixels + i) & face->format->Rmask;
			alpha >>=face->format->Rshift;
			alpha <<=face->format->Ashift;

			*((Uint32 *)face->pixels + i) = 0;

			*((Uint32 *)face->pixels + i) |= face->format->Rmask;
			*((Uint32 *)face->pixels + i) |= face->format->Gmask;
			*((Uint32 *)face->pixels + i) |= face->format->Bmask;

			*((Uint32 *)face->pixels + i) |= alpha;
		}
		break;
	}
	SDL_UnlockSurface(face);
}

sge_cdata *FeeImageManager::GetMask(char *filename)
{
	FeeImageList *list=FindImage(filename);
	if (!list) return NULL;
	return list->GetMask();
}

sge_cdata *FeeImageManager::GetFrameMask(char *filename, int frame)
{
	char *name=(char*)malloc(strlen(filename)+5);
	sprintf(name,"%s:%i",filename,frame);
	FeeImageList *list=FindImage(name);
	free(name);
	if (!list) return NULL;
	return list->GetMask();
}

SDL_Surface *FeeImageManager::CopyImage(SDL_Surface *face, char *name, int flags)
{
	FeeImageList *image;
	if (flags==IMAGE_SIMPLE)
		face=SDL_DisplayFormat(face);
	else
		face=SDL_DisplayFormatAlpha(face);
	image=FindImage(name);
	if (image!=NULL)
		RemoveImage(name);

	image=AddImage(face,name);
	return image->GetImage();
}

#ifdef HAVE_GL_H
GLuint FeeImageManager::GetTexture(SDL_Surface *face)
{
	if (!(imageList))
		return 0;

	FeeImageList *list = imageList;
	while (list)
	{
		if (list->GetImage()==face)
		{
			return list->GetTexture();
		}
		
		list=list->next;
	}

	return 0;
}

void FeeImageManager::UpdateTexture(SDL_Surface *face)
{
	if (!(imageList))
		return;

	FeeImageList *list = imageList;
	while (list)
	{
		if (list->GetImage()==face)
		{
			list->UpdateTexture();
		}
		
		list=list->next;
	}
}
#endif


void FeeImageManager::ReloadImage(SDL_Surface *face)
{
	if (!(imageList))
		return;

	FeeImageList *list = imageList;
	while (list)
	{
		if (list->GetImage()==face)
		{
			RemoveImage(list->name);
		}
		
		list=list->next;
	}
}
