// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// FeeManager.cpp: implementation of the FeeManager class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "FeeList.h"
#include "FeeSprite.h"
#include "FeeManager.h"
#include "FeeImageManager.h"
#include "FeeImageList.h"
#include "sge_misc.h"
#include "CON_console.h"
#include "string.h"
#include "GameOptions.h"
#ifdef macintosh
	#include "Movies.h"
#else
	#ifdef USE_SOUND
		#include "SDL_mixer.h"
	#endif
#endif

#ifdef HAVE_GL_H
#include "GLDynamicLoad.h"
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FeeManager::FeeManager()
{
	sge_Randomize(); // wo wenn nicht hier?
    targetSurface = NULL;
	bkgndSurface=NULL;
	spriteList=NULL;
	frame=0;
	soundSystemInitialized = false;
	imageManager=new FeeImageManager;
	options=new GameOptions(this);
	spriteCount=0;
	lastList=NULL;
	keystate=NULL;
#ifdef MACOSX
	options->levelOfDetail=4;
#else
	options->levelOfDetail=3;
#endif
}

FeeManager::~FeeManager()
{
#ifdef USE_SOUND
#ifndef macintosh //Quicktime muss nicht geschlossen werden
	if (soundSystemInitialized) 
		Mix_CloseAudio();
#endif
#endif
	FeeList *list=spriteList;
	FeeList *temp=spriteList;
	while (list)
	{
		temp=list;
		list = list->next;
		delete temp;
	}

	delete imageManager;
	delete options;
}

void FeeManager::Log(char *toLog)
{
	CON_ConOut(toLog);
}

void FeeManager::Update(float time)
{
	SDL_Rect pos = {0,0,maxX, maxY};
	if (options->useGL)
	{
	#ifdef HAVE_GL_H
		glLoadIdentity_priv();
		glOrtho_priv(0,maxX,maxY,0,-1,1);
		glClear_priv(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		pos.w=SafeSize(pos.w);
		pos.h=SafeSize(pos.h);
		glTranslatef_priv(0,0,0);

		glColor4ub_priv(255,255,255,255);

		GLuint tex=imageManager->GetTexture(bkgndSurface);
		glBindTexture_priv(GL_TEXTURE_2D, tex);		

		glBegin_priv(GL_QUADS);
			glTexCoord2f_priv(0.0f, 0.0f);
			glVertex3i_priv(0, 0, 0);

			glTexCoord2f_priv(1.0f, 0.0f);
			glVertex3i_priv(pos.w, 0, 0);

			glTexCoord2f_priv(1.0f, 1.0f);
			glVertex3i_priv(pos.w, pos.h, 0);

			glTexCoord2f_priv(0.0f, 1.0f);
			glVertex3i_priv(0, pos.h, 0);
		glEnd_priv();
	#endif
	}
	else
		SDL_BlitSurface(bkgndSurface, &pos, targetSurface, &pos);

	FeeList *list=spriteList;
	bool RemoveFlag=false;
	while (list)					// erster Durchlauf: Update
	{

		list->mySprite->Update(time);		
		list = list->next;
	}
	
	list=spriteList;
	while (list)					// zweiter Durchlauf: Kollisionen
	{
		{
			FeeList *list2=spriteList;
			while(list2)
			{
				if (list->mySprite!=list2->mySprite)
				{
					if ( list->mySprite->CollidesWith(list2->mySprite) )
					//if (Collides(list->mySprite->GetRect(),list2->mySprite->GetRect()))
					{
#ifdef _DEBUG
						char buf[250];
						sprintf(buf,"%s collides with %s\n",list->mySprite->name,list2->mySprite->name);
						ODS(buf);
#endif
						list2->mySprite->Hit(list->mySprite);					
					}
				}
				list2 = list2->next;
			}
		}
		list = list->next;
	}

	list=spriteList;
	FeeList *temp;
	while (list)				// dritter Durchlauf: loeschen/malen
	{
		if (list->mySprite->toBeRemoved)
		{
#ifdef _DEBUG
			char buf[250];
			sprintf(buf,"%s killed\n",list->mySprite->name);
			ODS(buf);
#endif
			temp=list->next;
			RemoveSprite(list->mySprite);
			list=temp;
		}
		else
		{
			list->mySprite->DrawTo(targetSurface);
			list=list->next;
		}
	}

	keystate = SDL_GetKeyState(NULL);
}

void FeeManager::Flip()
{
	if (options->useGL)
	{
	#ifdef HAVE_GL_H
		SDL_GL_SwapBuffers();
	#endif
	}
	else
		SDL_Flip(targetSurface);
}

void FeeManager::NewBkgnd(char *filename)
{
/*	if (targetSurface->format->BitsPerPixel==8)
	{
		char *palfile = (char*)malloc(MAX_PATH);
		sprintf(palfile, "%s.pal",filename);
		
		FILE *file;
		file=fopen(palfile,"r");
		if (file)
		{
			SDL_Color colors[256];
			int r,g,b,p;
			for(int i=0;i<256;i++)
			{
				fread(&r,1,1,file);
				fread(&g,1,1,file);
				fread(&b,1,1,file);
				fread(&p,1,1,file); // padding byte
				colors[i].r=r;
	            colors[i].g=g;
				colors[i].b=b;
			}
			SDL_SetPalette(targetSurface, SDL_LOGPAL|SDL_PHYSPAL, colors, 0, 256);
			fclose(file);
			CON_ConOut("Loaded palette %s.",palfile);
		}
	}*/

	bkgndSurface=imageManager->GetSimpleImage(filename);
	bkgndName=filename;	
	
	maxX=bkgndSurface->w;
	maxY=bkgndSurface->h;
}

void FeeManager::NewTransparentBkgnd(char *filename)
{
	bkgndSurface=imageManager->GetImage(filename);
	bkgndName=filename;
	maxX=bkgndSurface->w;
	maxY=bkgndSurface->h;
}


void FeeManager::AddSprite(FeeSprite *theSprite)
{
	lastList=NULL;
	spriteCount++;
	if (!(spriteList))
	{
		spriteList = new FeeList(theSprite);
		return;
	}
	FeeList *list = spriteList;
	spriteList = new FeeList(theSprite);
	spriteList->next = list;
}

void FeeManager::AddSpriteInFront(FeeSprite *theSprite)
{
	lastList=NULL;
	spriteCount++;
	if (!(spriteList))
	{
		spriteList = new FeeList(theSprite);
		return;
	}
	FeeList *list = spriteList;
	while (list->next)
		list=list->next;
	list->next = new FeeList(theSprite);
}

void FeeManager::RemoveSprite(FeeSprite *theSprite)
{
	if (!(spriteList))
		return;

	lastList=NULL;
	spriteCount--;
	FeeList *list = spriteList;
	FeeList *previous = NULL;
	while (list)
	{
		if (list->mySprite==theSprite)
		{
			if ((previous))
			{
				previous->next=list->next;
			}
			else
			{
				spriteList=list->next;
			}
			delete list;
			break;
		}
		previous=list; // schlagt mich!
		list=list->next;
	}
}

void FeeManager::InitializeSoundSystem()
{
#ifdef USE_SOUND
	if (!(soundSystemInitialized)) 
	{
		#ifdef macintosh // TODO: !!!PFUSCH!!!
			EnterMovies();	// Wenn Quicktime nicht installiert ist, kann man ohne Bedenken abstuerzen.
							// So dumm ist keiner, das zu deinstallieren.
			soundSystemInitialized = true;
		#else
			if (Mix_OpenAudio(11025, MIX_DEFAULT_FORMAT, 2, 1024) == 0) soundSystemInitialized = true;
		#endif
	}
#endif
}

void FeeManager::MoveSpriteToTop(FeeSprite *theSprite)
{
	if (!(spriteList)) // wenn keine Liste da ist, 
	{
		spriteList = new FeeList(theSprite); // hat sich das ganze erledigt.
		return;
	}
	
	FeeList *list = spriteList;
	FeeList *other = NULL;
	FeeList *previous = NULL;
	while (list) // passendes FeeList suchen
	{
		if (list->mySprite==theSprite)
		{
			if (list->next==NULL) return; // schon hinten.

			if ((previous))
			{
				previous->next=list->next; // rausnehmen
			}
			else
			{
				spriteList=list->next;
			}
			other=list; // aber speichern
			break;
		}
		previous=list; 
		list=list->next;
	}
	
	list = spriteList; // und hinten wiedereinfgen
	while (list->next)
		list=list->next;
	list->next = other;
	other->next=NULL; // das darf nmlich nicht in die Mitte der Liste zeigen
}					  // schlag mich!

SDL_Surface *FeeManager::GetBkgnd()
{
	return bkgndSurface;
}

int FeeManager::GetLOD()
{
	return options->levelOfDetail;
}

void FeeManager::SetLOD(int i)
{
	options->levelOfDetail=i;
}


// *******************************************************
//
// Sprite-Suchfunktionen
//
// *******************************************************

FeeSprite *FeeManager::GetSprite(int theKind)
{
	lastList=NULL;
	return GetNextSprite(theKind);
}

FeeSprite *FeeManager::GetNextSprite(int theKind)
{
	if (!(spriteList))
		return NULL;

	FeeList *list;

	if (lastList==NULL)
		list=spriteList;
	else
	{
		list=lastList->next;
		if (lastList->mySprite->myKind!=theKind)
			list=spriteList;
	}
	while (list)
	{
		if (list->mySprite->myKind==theKind)
		{
			lastList=list;
			return list->mySprite;
		}
		list=list->next;
	}
	lastList=NULL;
	return NULL;
}

bool FeeManager::SpriteExists(FeeSprite * theSprite)
{
	if (!(spriteList))
		return false;

	FeeList *list = spriteList;
	while (list)
	{
		if (list->mySprite==theSprite)
		{
			return true;
			break;
		}
		list=list->next;
	}

	return false;
}

SDL_Surface *FeeManager::InitializeScreen(int w, int h)
{
	if (targetSurface)
		SDL_FreeSurface(targetSurface);

	SDL_Surface *theScreen;

	int displayOptions=0;

	if (options->fullscreen) displayOptions|=SDL_FULLSCREEN;
	if (options->useHWSurf) displayOptions|=SDL_HWSURFACE|SDL_FULLSCREEN;
	//if (options->bpp==8) displayOptions|=SDL_HWPALETTE;
	
	if (options->useGL)
	{
#ifdef HAVE_GL_H
#ifdef GL_LINK_STATICALLY
		printf("Initializing OpenGL... (statically linked)\n");
#else
		printf("Initializing OpenGL... (dynamically linked)\n");
#endif

		if (!LoadGLFunctions())
		{
			printf("Dynamically loading GL functions failed. (%s)\n",SDL_GetError());
			return NULL;
		}


		SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
		SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
		SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
		SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 0 );
		SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

		//theScreen = SDL_SetVideoMode(pos.w, pos.h+15, 16,SDL_OPENGL);
		theScreen = SDL_SetVideoMode(w, h, 0,displayOptions|SDL_OPENGL);
#endif
	}
	else
	{
		theScreen = SDL_SetVideoMode(w, h, options->bpp,displayOptions);
	}
	
	targetSurface=theScreen;

	if (options->useGL)
	{
#ifdef HAVE_GL_H
        glClearColor_priv(0, 0, 0, 1);
        glClearDepth_priv(1);
		glEnable_priv(GL_TEXTURE_2D);
		glEnable_priv(GL_DITHER);
		glDisable_priv(GL_DEPTH_TEST);
		glHint_priv(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
        glMatrixMode_priv(GL_PROJECTION);
		glBlendFunc_priv(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable_priv(GL_BLEND);
		glDisable_priv(GL_ALPHA_TEST);
		glDisable_priv(GL_NORMALIZE);
		glShadeModel_priv(GL_SMOOTH);
        glLoadIdentity_priv();
        //glMatrixMode(GL_MODELVIEW);
        glClear_priv(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity_priv();
		CON_UseGL(true);
#endif
	}

	return theScreen;
}
