// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// FeeSprite.cpp: implementation of the FeeSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "FeeSprite.h"
#include "FeeManager.h"
#include "sge_collision.h"
#include "FeeHelper.h"
#include "FeeImageManager.h"
#include "FeeImageList.h"
#include "CON_console.h"
#include "GameOptions.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FeeSprite::FeeSprite(int x, int y, FeeManager *derManager,bool inFront)
{
	this->x=(float)x;
	this->y=(float)y;
	face = NULL;

	alpha=SDL_ALPHA_OPAQUE;

	myManager = derManager;
	if (inFront)
		myManager->AddSpriteInFront(this);
	else
		myManager->AddSprite(this);
	toBeRemoved=false;
	collisionMask=NULL;
	canCollide=true;
	updateCount=0;
	frame = 0;
	myKind = -1;
	name="NoName";
}

FeeSprite::~FeeSprite()
{
}

void FeeSprite::DrawTo(SDL_Surface *target)
{
	if (face==NULL) return;

    SDL_Rect pos1={0,0,face->w,face->h};
	SDL_Rect pos2;
	
	pos2=GetRect();

	if (myManager->options->useGL)
	{
		#ifdef HAVE_GL_H

		glColor4ub_priv(255,255,255,alpha);
	
		float w=(float)SafeSize(pos2.w);
		float h=(float)SafeSize(pos2.h);

        glTranslatef_priv(0,0,0);
        if (face)
		{
			GLuint tex=myManager->imageManager->GetTexture(face);
			glBindTexture_priv(GL_TEXTURE_2D, tex);
		}
		else
			glColor4ub_priv(255,255,255,alpha);

	
		glBegin_priv(GL_QUADS);
			//glColor3f(1.0f,1.0f,1.0f);
			glTexCoord2f_priv(0.0f, 0.0f);
			glVertex3f_priv(x, y, 0);

			//glColor3f(1.0f,0.0f,0.0f);
			glTexCoord2f_priv(1.0f, 0.0f);
			glVertex3f_priv(x+w, y, 0);

			//glColor3f(1.0f,1.0f,0.0f);
			glTexCoord2f_priv(1.0f, 1.0f);
			glVertex3f_priv(x+w, y+h, 0);

			//glColor3f(0.0f,0.0f,1.0f);
			glTexCoord2f_priv(0.0f, 1.0f);
			glVertex3f_priv(x, y+h, 0);
		glEnd_priv();
		#endif
	}
	else
	{
		if (face->format->alpha!=alpha)
			SDL_SetAlpha(face,SDL_SRCALPHA|SDL_RLEACCEL,alpha);
		if (SDL_BlitSurface(face,NULL,target,&pos2) == -2)
		{
			// Direct X hat uns den Speicher gelscht.
			myManager->imageManager->ReloadImage(face);
			face=NULL;
		}
	}
}


SDL_Rect FeeSprite::GetRect()
{
	SDL_Rect temp={0,0,0,0};
	if (face!=NULL)
	{
		temp.x=(int)x;
		temp.y=(int)y;
		temp.w=face->w;
		temp.h=face->h;
	}
	return temp;
}

void FeeSprite::InitFaceFromIMG(char *filename)
{
	if (collisionMask!=NULL)
	{
		sge_destroy_cmap(collisionMask);
		collisionMask=NULL;
	}

	face = myManager->imageManager->GetImage(filename);
	if (!face)
	{
		printf("Could not load %s (%s)",filename,SDL_GetError());
	}
}

int FeeSprite::CollidesWith(FeeSprite *theSprite)
{
	SDL_Rect rect1=GetRect();
	SDL_Rect rect2=theSprite->GetRect();
	
	if ((!(theSprite->canCollide))||(!(this->canCollide)))
		return false;
	
	if ((face!=NULL)&&(theSprite->face!=NULL))
	{
		if ( ( rect1.x >= rect2.x) && ( rect1.x <= (rect2.x+rect2.w) ) || ( (rect2.x >= rect1.x) && (rect2.x <= (rect1.x+rect1.w)) ) )
		{
			if ( (rect1.y >= rect2.y) && (rect1.y <= (rect2.y+rect2.h)) || ((rect2.y >= rect1.y) && (rect2.y <= (rect1.y+rect1.h)) ) ) 
			{
				// wenn das klappt, dann...
				// fngt der Spass erst richtig an.
				if (collisionMask==NULL)		// wenn noch keine Masken vorhanden sind,
				{
					if (SDL_LockSurface(face))         // der Spacken hat das natrlich vergessen >-(
						printf("Surface konnte nicht gelockt werden\n");	
					else
					{
						collisionMask=sge_make_cmap(face); // machen wir welche
						collisionMask->frame=0;
					}

					SDL_UnlockSurface(face);
				}

				if (theSprite->collisionMask==NULL)		 // dito
				{
					if (SDL_LockSurface(theSprite->face))
						printf("Surface konnte nicht gelockt werden\n");
					else
					{
						theSprite->collisionMask=sge_make_cmap(theSprite->face);
						theSprite->collisionMask->frame=0;
					}
					SDL_UnlockSurface(theSprite->face);
				}
				return sge_cmcheck(collisionMask,rect1.x,rect1.y,theSprite->collisionMask,rect2.x,rect2.y);
			}
		}
	}
	return false;
}


bool FeeSprite::DoAction(float time)
{
	updateCount+=time;
	if (updateCount>=1) 
	{
		updateCount-=1;
		return true;
	}
	return false;	
}

