// GameOptions.cpp: implementation of the GameOptions class.
//
//////////////////////////////////////////////////////////////////////

#include <string.h>
#include "FeeGlobal.h"
#include "GameOptions.h"
#include "CON_console.h"
#include "AIPlayerSprite.h"
#include "FeeManager.h"
#include "SnoopyGlobals.h"
#include "StatusSprite.h"
#ifdef WIN32
#include "winspecific.h"
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GameOptions::GameOptions(FeeManager *theManager)
{
	myManager=theManager;
	allowLanding=1;
	hitSettings=HIT_SETTINGS_GETS_WORSE;
	showAmmoBoard=AMMO_BOARD_NONE;
	withHM=0;
	useGL=0;
	bpp=16;
	fullscreen=0;
	useHWSurf=0;
	showFPS=0;
	dataPath=(char*)malloc(MAX_PATH);
	strcpy(dataPath,"./");
	filePath=(char*)malloc(MAX_PATH);
}

GameOptions::~GameOptions()
{
	free(filePath);
	free(dataPath);
}


bool GameOptions::Retrieve() // taken from chromium bsu
{
	bool	retVal = true;
	
	retVal=LoadOptions();

#ifdef WIN32
	if (!retVal)
	{
		ShowOptions();
	}
#endif
	
	OptionsChanged();

	StatusSprite *status=dynamic_cast<StatusSprite*>(myManager->GetSprite(STATUS_KIND));

	if (status)
		status->SetLevel(lastLevel);
	
	return retVal;
}

//----------------------------------------------------------
bool GameOptions::Store()
{
	bool retVal;
	char	configFilename[256];
	FILE	*file;

	AIPlayerSprite *computergegner;
	do
	{
		computergegner=dynamic_cast<AIPlayerSprite*>(myManager->GetSprite(PLAYER_KIND));
	} while (!computergegner);

	if (computergegner) useAI=(int)computergegner->IsActive();

	const char *homeDir = getenv("HOME");
#ifdef WIN32
	if (homeDir==NULL) homeDir = getenv("WINDIR"); // not nice, but this is at least always declared
#endif

	if(!homeDir)
		homeDir = "./";
		
	sprintf(configFilename, "%s/.snoopy", homeDir);
	file = fopen(configFilename, "w");
	if(file)
	{
		fprintf(file, "levelOfDetail %i\n",	levelOfDetail);
		fprintf(file, "useAIPlayer %i\n", useAI);
		fprintf(file, "lastLevel %i\n", dynamic_cast<StatusSprite*>(myManager->GetSprite(STATUS_KIND))->GetLevel());
		fprintf(file, "allowLanding %i\n", allowLanding);
		fprintf(file, "hitSettings %i\n", hitSettings);
		fprintf(file, "showAmmoBoard %i\n", showAmmoBoard);
		fprintf(file, "useHomingMissiles %i\n", withHM);
		fprintf(file, "useOpenGL %i\n", useGL);
		fprintf(file, "useHardwareSurface %i\n", useHWSurf);
		fprintf(file, "BitsPerPixel %i\n", bpp);
		fprintf(file, "Fullscreen %i\n", fullscreen);
		fprintf(file, "dataPath %s\n", dataPath);
		fprintf(file, "showFPS %i\n", showFPS);

		fclose(file);
		CON_ConOut("wrote config file (%s)\n", configFilename);
		retVal = true;
	}
	else
	{
		CON_ConOut("WARNING: could not write config file (%s)\n", configFilename);
		retVal = false;
	}
	
	return retVal;
}

void GameOptions::OptionsChanged()
{
	AIPlayerSprite *computergegner;
	do
	{
		computergegner=dynamic_cast<AIPlayerSprite*>(myManager->GetSprite(PLAYER_KIND));
	} while ((!computergegner));

	if (bpp==8) CON_ConsoleAlpha(0);

	if (computergegner)
	{
		if (useAI) 
		{
			if (!computergegner->IsActive())
				computergegner->Activate();
		}
		else
		{
			if (computergegner->IsActive())
				computergegner->Activate();
		}
	}
}

int GameOptions::LoadOptions()
{
	int		i;
	int		numLines;
	bool	retVal = true;
	char	configFilename[256];
	FILE	*file;
	int		tmp;
	
	const char *homeDir = getenv("HOME");
#ifdef WIN32
	if (homeDir==NULL) homeDir = getenv("WINDIR"); // not nice, but this is at least always declared
#endif

	char	configStrings[32][64];

	if(!homeDir)
		homeDir = "./";

	sprintf(configFilename, "%s/.snoopy", homeDir);
	file = fopen(configFilename, "r");
	if(file)
	{
		i = numLines = 0;
		while( fgets(configStrings[i], 64, file) )
			i++;
		numLines = i;
		for(i = 0; i < numLines; i++)
		{
			if(strncmp(configStrings[i], "levelOfD", 8) == 0) { sscanf(configStrings[i], "levelOfDetail %i\n", &levelOfDetail); }
			if(strncmp(configStrings[i], "useAIPla", 8) == 0) { sscanf(configStrings[i], "useAIPlayer %i\n", &useAI); }
			if(strncmp(configStrings[i], "lastLeve", 8) == 0) { sscanf(configStrings[i], "lastLevel %i\n", &lastLevel); }
			if(strncmp(configStrings[i], "allowLan", 8) == 0) { sscanf(configStrings[i], "allowLanding %i\n", &allowLanding); }
			if(strncmp(configStrings[i], "hitSetti", 8) == 0) { sscanf(configStrings[i], "hitSettings %i\n", &hitSettings); }
			if(strncmp(configStrings[i], "showAmmo", 8) == 0) { sscanf(configStrings[i], "showAmmoBoard %i\n", &showAmmoBoard); }
			if(strncmp(configStrings[i], "useHomin", 8) == 0) { sscanf(configStrings[i], "useHomingMissiles %i\n", &withHM); }
			if(strncmp(configStrings[i], "useOpenG", 8) == 0) { sscanf(configStrings[i], "useOpenGL %i\n", &useGL); }
			if(strncmp(configStrings[i], "useHardw", 8) == 0) { sscanf(configStrings[i], "useHardwareSurface %i\n", &useHWSurf); }
			if(strncmp(configStrings[i], "BitsPerP", 8) == 0) { sscanf(configStrings[i], "BitsPerPixel %i\n", &bpp); }
			if(strncmp(configStrings[i], "Fullscre", 8) == 0) { sscanf(configStrings[i], "Fullscreen %i\n", &fullscreen); }
			if(strncmp(configStrings[i], "dataPath", 8) == 0) { sscanf(configStrings[i], "dataPath %s\n", dataPath); }
			if(strncmp(configStrings[i], "showFPS", 7) == 0) { sscanf(configStrings[i], "showFPS %i\n", &showFPS); }
		}
		CON_ConOut("read config file %s.\n", configFilename);
	}
	else
	{
		CON_ConOut("WARNING: could not read config file (%s)\n", configFilename);
		retVal=false;
	}

#ifndef HAVE_GL_H
	useGL=0;
#endif

	return retVal;
}

char *GameOptions::GetDataPath(char *filename)
{
	sprintf(filePath,"%s/%s",dataPath,filename);
	return filePath;
}

