// HCAIPlayerSprite.cpp: implementation of the HCAIPlayerSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "FeeGlobal.h"
#include "SnoopyGlobals.h"
#include "FeeSprite.h"
#include "FeeManager.h"
#include "MovingSprite.h"
#include "PlayerSprite.h"
#include "AIPlayerSprite.h"
#include "HCAIPlayerSprite.h"
#include "AIaction.h"
#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HCAIPlayerSprite::HCAIPlayerSprite(int x, int y, int kind, FeeManager *derManager):AIPlayerSprite(x, y, kind, derManager)
{
	ActionQueue=NULL;
	threatCountdown=0;
	actionCount=0;
	
	PlayerSprite *target;
	PlayerSprite *me=dynamic_cast<PlayerSprite*>(this);
	target=dynamic_cast<PlayerSprite*>(myManager->GetSprite(PLAYER_KIND));
	if (target==me) target=dynamic_cast<PlayerSprite*>(myManager->GetNextSprite(PLAYER_KIND));
	
	AddAction(AI_ATTACK_PLAYER,target);
	AddAction(AI_AVOID_GROUND,NULL);
	AddAction(AI_GET_PARACHUTE,NULL);
	AddAction(AI_LAND,target);
	
	myManager->Log(":-) Funny scripted hardcoded AI");
}

HCAIPlayerSprite::~HCAIPlayerSprite()
{
	while (ActionQueue)
		RemoveAction(ActionQueue);
}


void HCAIPlayerSprite::Update(float time)
{
	if (active)
	{
		if (ActionQueue!=NULL)
		{
			AIaction *action = ActionQueue;
			AIaction *bestAction = ActionQueue;
			AIaction *lastAction = NULL;
			
			while (action)	// fuer jede Aktion
			{
				if (GetPriority(bestAction)<GetPriority(action)) bestAction=action; // Aktion mit hchster Prioritt whlen
				
				if ((action->Target!=NULL)&&(!myManager->SpriteExists(action->Target)))
				{
				    action->toDelete=true;				
				}
				
				action=action->next;
			}
			
			action=bestAction;
			
			switch (action->Command)
			{
			case AI_ATTACK_PLAYER:
				{
					MovingSprite *target=dynamic_cast<MovingSprite*>(action->Target);
					if (target)
					{
						int distance=GetDistance(target);
						int dir=GetTurnDirection(target, distance/(speed+ARROW_SPEED));
						
						if (dir)
						{
							SetAngle(dir); // hindrehen oder
							SetSpeed(HIGH_SPEED);
							if (GetTargetAngle(target,distance/(speed+5))==4)
								if (GetDistance(target)>40)
									DropBomb();
						}
						else
						{
							action->Countdown++;
							if (action->Countdown>=4)
							{
								FireShot(); // schiessen
								action->Countdown=0;
							}
						}
						if (distance<80)
							SetSpeed(LOW_SPEED);
						else
							SetSpeed(HIGH_SPEED);
					}
					break;
				}
			case AI_AVOID_GROUND:
				{
					SetAngle(GetTurnDirection(x,0));	// nach oben drehen
					if (y>(myManager->maxY-BODEN-40))
						SetSpeed(LOW_SPEED);
					else
						SetSpeed(HIGH_SPEED);
					break;
				}
			case AI_GET_PARACHUTE:
				{
					FeeSprite *target=action->Target;
					int dir=0;

					if (!myManager->SpriteExists(target)) target=NULL;
					
					if (!target)
					{
						FeeSprite *object=myManager->GetSprite(PARA_KIND);

						while (object!=NULL)
						{
							if (object)
								if ((object->y+20)<y)
								{
									target=object;
									object=NULL;
								}
							object=myManager->GetNextSprite(PARA_KIND);
						}
					}


					if (target)
						dir=GetTurnDirection(target->x,target->y+29);
					else
						if (myManager->GetSprite(PARA_KIND))
							dir=GetTurnDirection(x,myManager->maxY);
						else
							dir=GetTurnDirection(x,myManager->maxY/3);
						
						SetAngle(dir);
						SetSpeed(HIGH_SPEED);
						break;
				}
			case AI_DODGE:
				{
					FeeSprite *target=action->Target;
					int dir=0;

					if ((!myManager->SpriteExists(target))&&(target)) 
					{
						action->toDelete=true;
					}
					else
					{
						dir=GetTurnDirection(target);
						if (dir==INCR_ANGLE) dir=DECR_ANGLE;
						if (dir==DECR_ANGLE) dir=INCR_ANGLE;
						SetAngle(dir);
					}
					break;
				}
			case AI_LAND:
				{
					MovingSprite *target=dynamic_cast<MovingSprite*>(action->Target);

					int dir=0;

					int max=5;
					int min=3;

					if (target)
					{
						int distance=GetDistance(target);
						dir=GetTurnDirection(target, distance/(speed+ARROW_SPEED));
						
						if (!dir)
							FireShot(); // schiessen

						if (target->x<x) max=7; // nur so aus scheiss versuchen wir, in Richtung Gegner zu steuern (tendenziell)
						if (target->x>x) min=1;
					}
					

					if (y<(myManager->maxY-BODEN-50))
					{
						if (fallDirection==DIR_CLOCKWISE) // wenn wir abstrzen, versuchen wir, mglichst schnell niedrig zu kommen.
							if ((angle>max)||(angle<min))
								dir=INCR_ANGLE;
							else
								dir=DECR_ANGLE;
						else
							if ((angle>max)||(angle<min))
								dir=DECR_ANGLE;
							else
								dir=INCR_ANGLE;
					}
					else // wenn wir dann weit unten sind,
					{
						SetSpeed(LOW_SPEED);
						if (fallDirection==DIR_CLOCKWISE) // wenn wir abstrzen, versuchen wir, mglichst schnell niedrig zu kommen.
							if ((angle>15)||(angle<2))
								dir=DECR_ANGLE;
							else 
								dir=INCR_ANGLE;
						else
							if ((angle>15)||(angle<2))
								dir=INCR_ANGLE;
							else
								dir=DECR_ANGLE;
					}						
					SetAngle(dir);
				}

			default:
				;
			}
		}
	}
	
	AIaction *action=ActionQueue;
	AIaction *temp;
	while (action)				// loeschen
	{
		if (action->toDelete)
		{
#ifdef _DEBUG
			char buf[250];
			sprintf(buf,"Killed with Command %i\n",action->Command);
			ODS(buf);
#endif
			temp=action->next;
			RemoveAction(action);
			action=temp;
		}
		else
		{
			action=action->next;
		}
	}

	threatCountdown++;
	if (threatCountdown>=2)
	{
		CheckForThreats();
		threatCountdown=0;
	}
	
	AIPlayerSprite::Update(time);
}

void HCAIPlayerSprite::Hit(FeeSprite *hitBy)
{
	if (active)
	{
		
	}
	AIPlayerSprite::Hit(hitBy);
}

int HCAIPlayerSprite::GetPriority(AIaction *action)
{
	switch (action->Command)
	{
	case AI_ATTACK_PLAYER:
		{
			PlayerSprite *Target=dynamic_cast<PlayerSprite*>(action->Target);
			if (Target)
			{
				if (Target->GetDead()||(Target->GetHits()>2))
					return AI_PRIORITY_SUSPENDED;
				else if (Target->GetHits()>0)
					return AI_PRIORITY_LESS_THAN_MEDIUM;
				else if (Target->y>(myManager->maxY-BODEN-30))
					return AI_PRIORITY_LOW;
				else
					return AI_PRIORITY_MEDIUM;
			}
			else
				action->toDelete=true;
			break;
		}
	case AI_AVOID_GROUND:
		{
			if (y>(myManager->maxY-BODEN-30)) 
				return AI_PRIORITY_HIGHEST;
			else if (y>(myManager->maxY-BODEN-50)) 
				return AI_PRIORITY_HIGH;
			else return AI_PRIORITY_LOW;
			break;
		}
	case AI_GET_PARACHUTE:
		{
			if ((munition[ARROW_KIND]==0)
				&&(munition[MG_KIND]==0)
				)
				return AI_PRIORITY_LESS_THAN_HIGH;
			else
				return AI_PRIORITY_LESS_THAN_MEDIUM;
			break;
			if (action->Target)
				if (!myManager->SpriteExists(action->Target))
					action->Target=NULL;
		}
	case AI_DODGE:
		{
			if (myManager->SpriteExists(action->Target)&&(action->Target))
			{
				int distance=GetDistance(action->Target);
				if (distance<140)
					return AI_PRIORITY_HIGH;
				else if (distance<200)
					return AI_PRIORITY_MEDIUM;
				else
				{
					action->toDelete=true;
					return AI_PRIORITY_SUSPENDED;
				}
			}
			else
			{
				action->toDelete=true;
				action->Target=NULL;
				return AI_PRIORITY_SUSPENDED;
			}
			break;
		}
	case AI_LAND:
		{
			if (hits>0)
				return AI_PRIORITY_HIGHEST;
			else
				return AI_PRIORITY_SUSPENDED;
		}
	}
	return AI_PRIORITY_SUSPENDED;
}

void HCAIPlayerSprite::AddAction(int theCommand, FeeSprite * target)
{
	AIaction *action=ActionQueue;
	ActionQueue=new AIaction(theCommand, target);
	ActionQueue->next=action;
	actionCount++;
}

void HCAIPlayerSprite::RemoveAction(AIaction *theAction)
{
	if (!(ActionQueue))
		return;

	actionCount--;

	AIaction *list = ActionQueue;
	AIaction *previous = NULL;
	while (list)
	{
		if (list==theAction)
		{
			if ((previous))
			{
				previous->next=list->next;
			}
			else
			{
				ActionQueue=list->next;
			}
			delete list;
			break;
		}
		previous=list; // schlagt mich!
		list=list->next;
	}
}

int HCAIPlayerSprite::GetTurnDirection(FeeSprite *target)
{
	double temp;
	int ydif=y-target->y;
	int xdif=x-target->x;
	
	
	if (abs(xdif)>(myManager->maxX/2)) xdif*=-1;
	
	temp=(((atan2(ydif,xdif)+PI)*8)/PI);
	
	int angle=(((int)temp)%16);
	int angle2=angle;
	
	ydif=abs(y-target->y);
	xdif=abs(x-target->x);
	
	int dist=(int)(double)sqrt((pow(xdif,2)+pow(ydif,2)));
	
	if (angle!=GetAngle())
	{
		if (angle < 8)
			angle += 16;
		else
			angle -= 16;
		if (abs(angle - GetAngle()) > abs(angle2 - GetAngle()))
			angle = angle2;
		
		if (angle > GetAngle())
			return INCR_ANGLE;
		else
			return DECR_ANGLE;
	}
	return 0;
}

int HCAIPlayerSprite::GetTurnDirection(int newX, int newY)
{
	double temp;
	int ydif=y-newY;
	int xdif=x-newX;
	
	if (abs(xdif)>(myManager->maxX/2)) xdif*=-1;
	
	temp=(((atan2(ydif,xdif)+PI)*8)/PI);
	
	int angle=(((int)temp)%16);
	int angle2=angle;
	
	ydif=abs(y-newY);
	xdif=abs(x-newX);
	
	int dist=(int)(double)sqrt((pow(xdif,2)+pow(ydif,2)));
	
	if (angle!=GetAngle())
	{
		if (angle < 8)
			angle += 16;
		else
			angle -= 16;
		if (abs(angle - GetAngle()) > abs(angle2 - GetAngle()))
			angle = angle2;
		
		if (angle > GetAngle())
			return INCR_ANGLE;
		else
			return DECR_ANGLE;
	}
	return 0;
}

int HCAIPlayerSprite::GetTurnDirection(MovingSprite *target, int time)
{
	int posX=target->x+(target->speedX*time);
	int posY=target->y+(target->speedY*time);
	return GetTurnDirection(posX,posY);
}

int HCAIPlayerSprite::GetTargetAngle(int newX, int newY)
{
	double temp;
	int ydif=y-newY;
	int xdif=x-newX;
	
	if (abs(xdif)>(myManager->maxX/2)) xdif*=-1;
	
	temp=(((atan2(ydif,xdif)+PI)*8)/PI);
	
	return (((int)temp)%16);
}

int HCAIPlayerSprite::GetTargetAngle(MovingSprite *target, int time)
{
	int posX=target->x+(target->speedX*time);
	int posY=target->y+(target->speedY*time);
	return GetTargetAngle(posX,posY);
}

int HCAIPlayerSprite::GetDistance(FeeSprite *target)
{
	int ydif=y-target->y;
	int xdif=x-target->x;
	
	return (int)(double)sqrt((pow(xdif,2)+pow(ydif,2)));
}

void HCAIPlayerSprite::CheckForThreats()
{
	FeeSprite *object=myManager->GetSprite(ARROW_KIND);
	FeeSprite *target=NULL;
	int minDist=200;
		
	while(object)
	{
		int distance=GetDistance(object);
		if (distance<minDist) 
		{
			minDist=distance;
			target=object;
		}
		
		object=myManager->GetNextSprite(ARROW_KIND);
	}
			
	object=myManager->GetSprite(MG_KIND);
		
	while(object)
	{
		int distance=GetDistance(object);
		if (distance<minDist) 
		{
			minDist=distance;
			target=object;
		}

		object=myManager->GetNextSprite(MG_KIND);
	}
				
	object=myManager->GetSprite(HM_KIND);

	while(object)
	{
		int distance=GetDistance(object);
		if (distance<minDist) 
		{
			minDist=distance;
			target=object;
		}
			
		object=myManager->GetNextSprite(HM_KIND);
	}
					
	if (target) 
	{
		AIaction *action=FindAction(AI_DODGE);

		if (action)
			action->Target=target;
		else
			AddAction(AI_DODGE, target);
	}
}

AIaction *HCAIPlayerSprite::FindAction(int type)
{
	if (!(ActionQueue))
		return NULL;

	AIaction *list = ActionQueue;
	while (list)
	{
		if (list->Command==type)
		{
			return list;
			break;
		}
		list=list->next;
	}
	return NULL;
}
