// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// PAIPlayerSprite.cpp: implementation of the PAIPlayerSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "PlayerSprite.h"
#include "PAIPlayerSprite.h"
#include "AIPlayerSprite.h"
#include "CON_console.h"
#include "FeeManager.h"
#include "GameOptions.h"
#ifdef HAVE_LIBDL
#ifdef __cplusplus
extern "C" {
#endif

#include <dlfcn.h>

#ifdef __cplusplus
};
#endif

#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


PAIPlayerSprite::PAIPlayerSprite(int x, int y, int kind, FeeManager *derManager, char *library):AIPlayerSprite(x,y,kind,derManager)
{
	// Hier sollten die Funktionen 
	// Update(PAIPlayerSprite *me, FeeManager *myManager)
	// und 
	// Hit(PAIPlayerSprite *me, FeeManager *myManager, FeeSprite *hitBy)
	// aus char *library lokalisiert und gemappt werden.
	// (mit dlopen()) in Unices, LoadLibrary/GetProcAdress in Win32.
    target1=NULL;
	target2=NULL;
	actionCount1=0;
	actionCount2=0;
	actionCount3=0;
	ExternalUpdate=NULL;
	ExternalHit=NULL;
	GetInformation=NULL;
#if defined(WIN32)
	libraryHandle=LoadLibrary(library);
	if (libraryHandle==NULL)
		CON_ConOut("Library %s not loaded.",library);
	ExternalUpdate=(TExternalUpdate)GetProcAddress(libraryHandle,"ExternalUpdate");
	if (!ExternalUpdate)
		CON_ConOut("Function ExternalUpdate not found.");
	ExternalHit=(TExternalHit)GetProcAddress(libraryHandle,"ExternalHit");
	if (!ExternalHit)
		CON_ConOut("Function ExternalHit not found.");
	GetInformation=(TGetInformation)GetProcAddress(libraryHandle,"GetInformation");
	if (!GetInformation)
		CON_ConOut("Function GetInformation not found.");
#elif defined(HAVE_LIBDL)
	libraryHandle=dlopen(library,RTLD_LAZY);
	if (!libraryHandle) 
		CON_ConOut("Library %s not loaded. (%s)",library,dlerror());
	ExternalUpdate=(TExternalUpdate)dlsym(libraryHandle,"ExternalUpdate");
	if (!ExternalUpdate)
        {
            ExternalUpdate=(TExternalUpdate)dlsym(libraryHandle,"_ExternalUpdate");
            if (!ExternalUpdate)
                CON_ConOut("%s",dlerror());
        }
	ExternalHit=(TExternalHit)dlsym(libraryHandle,"ExternalHit");
	if (!ExternalHit)
        {
            ExternalHit=(TExternalHit)dlsym(libraryHandle,"_ExternalHit");
            if (!ExternalHit)
                CON_ConOut("%s",dlerror());
        }
	GetInformation=(TGetInformation)dlsym(libraryHandle,"GetInformation");
	if (!GetInformation)
        {
            GetInformation=(TGetInformation)dlsym(libraryHandle,"_GetInformation");
            if (!GetInformation)
                CON_ConOut("%s",dlerror());
        }
#else
	CON_ConOut("PluggableAI player interface has not yet been implemented for this platform.");
#endif

	if (GetInformation)
	{
		char name[250];
		int version;
		GetInformation((char*)&name,sizeof(name),&version);
		CON_ConOut("%s, for version %i",name,version);
	}

}

PAIPlayerSprite::~PAIPlayerSprite()
{
#if defined(HAVE_DLOPEN)||defined(WIN32)
	// Hier muss die Library gefreed werden.
	if (libraryHandle)
	{
#if defined(WIN32)
		FreeLibrary(libraryHandle);
#elif defined(HAVE_LIBDL)
		dlclose(libraryHandle);
#endif
	}
#endif
}



void PAIPlayerSprite::Update(float time)
{
	if ((ExternalUpdate) && active)
	{
		ExternalUpdate(this, myManager);
	}
	PlayerSprite::Update(time);
}

void PAIPlayerSprite::Hit(FeeSprite *hitBy)
{
	if ((ExternalHit) && active)
        {
		ExternalHit(this, myManager, hitBy);
	}
	PlayerSprite::Hit(hitBy);
}