// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// ParaSprite.cpp: implementation of the ParaSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "MovingSprite.h"
#include "ParaSprite.h"
#include "SnoopyGlobals.h"
#include <math.h>
#include "sge_misc.h"
#include "FeeManager.h"
#include "ExplosionSprite.h"
#include "ShotSprite.h"
#include "FeeImageManager.h"
#include "FeeSound.h"
#include "FeeImageList.h"
#include "GameOptions.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParaSprite::ParaSprite(int x, int weaponkind, FeeManager *theManager):MovingSprite(x, y, theManager, false)
{
	myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/Fallschirm.tga"),12);
	myKind=PARA_KIND;
	weapon=weaponkind;
	age=0;
	state=STATE_FLYING;
	name="Reinforcement";
	y=-35;
	this->x=x;
	frame=weapon*3;
	InitFace();
	speedX=0;
	speedY=1;
}

ParaSprite::~ParaSprite()
{
}

void ParaSprite::Update(float time)
{
	MovingSprite::Update(time);

	if (DoAction(time))
	{
		if (state==STATE_FALLING)
		{
			speedX=0;
			if ((speedY++)>FALL_SPEED) speedY=FALL_SPEED; // gleichfoermig beschleunigte Bewegung
		}

		if (((int)y%4)==0)
		{
			if (speedX == 0)					// abgeschrieben aus Fallschirm.p
				speedX = sge_Random(0,2)-1;		// ICH verstehe das nicht.
			else if (speedX==1)
				speedX = min(sge_Random(0,2),1);
			else
				speedX = max(sge_Random(0,2)-2,-1);
		}

		if (x > myManager->maxX) x += -29 -myManager->maxX; // an der Seite rausgeflogenen Fallschirm
		else if (x < (-29)) x = myManager->maxX +29 + x;    // auf die andere Seite bringen			

		if (state==STATE_LANDED)
		{
			speedY=0;
			speedX=0;
			age++;
		}

		if (age>130)
		{
			alpha-=10;
			if (alpha<10)
				toBeRemoved=true;
		}
	}
	if (y>myManager->maxY) toBeRemoved=true;
}

void ParaSprite::Hit(FeeSprite *hitBy)
{
	if (toBeRemoved) return;
	if ((hitBy->myKind==GROUND_KIND)&&(state!=STATE_LANDED))
	{
		if (state==STATE_FALLING)
		{
			state=STATE_LANDED;
			Explode();
		}
		else
		{
			state=STATE_LANDED;
			frame=weapon*3+2;		
			y+=2;
		}
		InitFace();
	}
	if ((hitBy->myKind==ARROW_KIND)||(hitBy->myKind==MG_KIND)||(hitBy->myKind==HM_KIND))
	{
		if (hitBy->y>(y+16)) // unterer Teil getroffen
		{
			Explode();
		}
		else				// Fallschirm getroffen
		{
            theStartBombSound->Play();
			state=STATE_FALLING;
			frame=weapon*3+1;
			InitFace();
		}
	}
	if (hitBy->myKind==EXPLO_KIND)
		Explode(); // da gibts nichts...
	if (hitBy->myKind==PLAYER_KIND)
	{
		if (!PlayerHitParachute(hitBy)) // unterer Teil; aufgefangen
		{
                        theReceivedSound->Play();
			toBeRemoved=true;
		}
		else				// Fallschirm getroffen
		{
			state=STATE_FALLING;
			frame=weapon*3+1;
			InitFace();
		}
	}
}

void ParaSprite::InitFace()
{
	face=myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/Fallschirm.tga"),frame);
	collisionMask=myManager->imageManager->GetFrameMask(myManager->options->GetDataPath("Grafiken/Fallschirm.tga"),frame);
}

void ParaSprite::Explode()
{
	int posX,posY;
	toBeRemoved=true;
	int i=0;
	new ExplosionSprite(x+14,y+14,myManager);

	if (weapon==HM_KIND) weapon=BOMB_KIND;        
	if (weapon==BOMB_KIND) return;
	
	// folgende Annahme: die Explode-Prozedur wird nur
	// von Hit abgefragt. Zu diesem Zeitpunkt ist es gefahrlos moeglich,
	// neue Sprites in die Liste einzufuegen.
	for (;i<16;i++)
	{
		if (weapon==ARROW_KIND) i++;	// nur halb so viele Pfeile schieen.
		posX=x+14+(cos(double(i)/8*PI)*45);
		posY=y+14+(sin(double(i)/8*PI)*45);
		if ((posY<(myManager->maxY))&&(posY>0))  // sonst schiessen wir nach unten aus dem Bildschirm raus :-)
			new ShotSprite(posX,posY,weapon,i,HIGH_SPEED,NULL,myManager);
	}
}

bool ParaSprite::PlayerHitParachute(FeeSprite *hitBy)
{
	if (state==STATE_LANDED) return false;
	if (y>(myManager->maxY-BODEN-10)) return false;
	if (sge_get_cy()<(y+18)) return true;
	return false;
}