// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

#include "SnoopyGlobals.h"
#include "FeeGlobal.h"
#include "PumpkinSprite.h"
#include "ShotSprite.h"
#include "FeeManager.h"
#include "ExplosionSprite.h"
#include "FeeSound.h"
#include <math.h>
#include "PlayerSprite.h"
#include "sge_misc.h"
#include "FeeImageManager.h"
#include "FlyingPartSprite.h"
#include "GameOptions.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PumpkinSprite::PumpkinSprite(int x, int y, FeeManager *derManager):MovingSprite(x,y, derManager)
{
	speed = LOW_SPEED;
	angle = 12;
	myKind=PUMPKIN_KIND;
	currentTarget = NULL;
	name="Great Pumpkin";
	thePumpkinSound->Play();
	hits = 0;
	einblenden = 0;
	myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/Pumpkin.tga"),2);

	normalFace = myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/Pumpkin.tga"),frame);
	alphaFace = myManager->imageManager->CopyImage(normalFace,"PumpkinAlphaFace");

	collisionMask = myManager->imageManager->GetFrameMask(myManager->options->GetDataPath("Grafiken/Pumpkin.tga"),frame);

}

PumpkinSprite::~PumpkinSprite()
{
}

void PumpkinSprite::Update(float time)
{
	if (DoAction(time))
	{
		einblenden++;
		SelectFace();
		if (einblenden <= 40) 
		{
			if (myManager->options->useGL)
				alpha = 255 / 40 * einblenden;
			else
				SelectFace();

			speed=STOP_SPEED;
			ComputeSpeed();

			return; // Nicht bewegen waehrend des Erscheinens
		}
		else
			alpha=SDL_ALPHA_OPAQUE;

		if (currentTarget)
		{
			angle = (atan2(-(y+32)+currentTarget->y, -(x+43) + currentTarget->x) / PI * 8);
			speed = HIGH_SPEED+1;	//Na gut, bei dieser Beweglichkeit ist das schnell genug
		}
		else
		{
			speed = LOW_SPEED;
			switch (sge_Random(0,2))
			{
				case 0: angle += 1;
				break;
				case 1: angle -= 1;
				break;
			}
			if (angle < 10) angle = 10;
			if (angle > 14) angle = 14;
			if ((x < 0) && (angle < 12)) angle = 12;					// Kuerbis soll nicht nach links oder rechts
			if ((x > myManager->maxX) && (angle > 12)) angle = 12;		// abhauen koennen
		}
		ComputeSpeed();
	}

	MovingSprite::Update(time);

	if (y < (-150)) toBeRemoved = true;
}

void PumpkinSprite::Hit(FeeSprite *hitBy)
{
	bool previousState = (currentTarget != NULL);	// Damit das Face nur bei Aenderungen geupdated werden muss
	if (hitBy->myKind == PLAYER_KIND)
	{
		// die stuerzen jetzt ab, also totlachen:
		thePumpkinSound->Play();
		if ((dynamic_cast<PlayerSprite*>(hitBy)) == currentTarget) currentTarget = NULL;
	}
	if (hitBy->myKind == ARROW_KIND)
	{
		hits += 2;
		thePumpkinHitSound->Play();
		currentTarget = (dynamic_cast<ShotSprite*>(hitBy))->Owner();
	}
	if (hitBy->myKind == MG_KIND)
	{
		hits += 1;
		thePumpkinHitSound->Play();
		currentTarget = (dynamic_cast<ShotSprite*>(hitBy))->Owner();
	}
	if (hits >= 15) //auch der Pumpkin ist nicht unverwundbar
	{
		toBeRemoved = true;
		theExplosionSound->Play();
		if (currentTarget->kills > 0) currentTarget->kills--; // Es muss ja einen Anreiz geben
		new ExplosionSprite(x+25,y+20,myManager);
		FlyingPartSprite::SpawnFlyingParts(x+25, y+20, myManager->options->GetDataPath("Grafiken/PumpkinParts.tga"), 5, myManager); // Teile losfliegen lassen
		canCollide = false;	//wichtig
	}
	if (previousState != (currentTarget != NULL)) SelectFace();	// Face updaten, wenn sich etwas veraendert hat
}

void PumpkinSprite::SelectFace()
{
	if (currentTarget)
		frame = 1;
	else
		frame = 0;
	if (!myManager->options->useGL)
	{
		if (einblenden <= 40) 
		{
			alphaFace = myManager->imageManager->CopyImage(normalFace,"PumpkinAlphaFace");
			if (myManager->GetLOD()>=3)	// nicht zu viel rechnen...
				SetAlphaFace(255 / 40 * einblenden);
			face=alphaFace;
		}
		else
			face = myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/Pumpkin.tga"),frame);
	}
	else
	{
		normalFace = myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/Pumpkin.tga"),frame);
		face=normalFace;
	}
}

void PumpkinSprite::SetAlphaFace(int alpha)
{
	int32 i;
	SDL_LockSurface(alphaFace);
	Uint32 oldAlpha;
	Uint32 newAlpha;
	switch (alphaFace->format->BytesPerPixel)	// und neuen Alphawert setzen
	{
	case 1:
		for (i = 0; i < (alphaFace->w * alphaFace->h); i++) // FIXME: Pitch einbeziehen!
		{
			oldAlpha = *((Uint8 *)alphaFace->pixels + i) & alphaFace->format->Amask;
			
			newAlpha = (float)oldAlpha / 255 * alpha;	//neuen Alphawert berechnen
			newAlpha &= alphaFace->format->Amask; //nur zur Sicherheit
			
			*((Uint8 *)alphaFace->pixels + i) ^= oldAlpha;
			*((Uint8 *)alphaFace->pixels + i) |= newAlpha;
		}
		break;
		
	case 2:
		for (i = 0; i < (alphaFace->w * alphaFace->h); i++)			
		{
			oldAlpha = *((Uint16 *)alphaFace->pixels + i) & alphaFace->format->Amask;
			
			newAlpha = (float)oldAlpha / 255 * alpha;	//neuen Alphawert berechnen
			newAlpha &= alphaFace->format->Amask; //nur zur Sicherheit
			
			*((Uint16 *)alphaFace->pixels + i) ^= oldAlpha;
			*((Uint16 *)alphaFace->pixels + i) |= newAlpha;
		}
		break;
		
	case 4:
		for (i = 0; i < (alphaFace->w * alphaFace->h); i++)			
		{
			oldAlpha = *((Uint32 *)alphaFace->pixels + i) & alphaFace->format->Amask;
			
			newAlpha = (float)oldAlpha / 255 * alpha;	//neuen Alphawert berechnen
			newAlpha &= alphaFace->format->Amask; //nur zur Sicherheit
			
			*((Uint32 *)alphaFace->pixels + i) ^= oldAlpha;
			*((Uint32 *)alphaFace->pixels + i) |= newAlpha;
		}
		break;
	}
	SDL_UnlockSurface(alphaFace);
}

