// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// SchroederSprite.cpp: implementation of the SchroederSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "FeeSprite.h"
#include "SchroederSprite.h"
#include "FeeManager.h"
#include "SnoopyGlobals.h"
#include "sge_misc.h"
#include "CON_console.h"
#include "ShotSprite.h"
#include "PlayerSprite.h"
#include "NoteSprite.h"
#include "FlyingPartSprite.h"
#include "FeeImageManager.h"
#include "FeeSound.h"
#include "GameOptions.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SchroederSprite::SchroederSprite(FeeManager *theManager):FeeSprite(-24,280,theManager, false)
{
	myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/Schroeder.tga"),16);
	myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/Piano.tga"),2);
	myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/PianoParts.tga"),20);
	
	name="Schroeder";
	myKind=SCHROEDER_KIND;	
	state=SCHROEDER_SLEEPING;
	processCount=0;
	curseAt=NULL;
	InitFace();
}

SchroederSprite::~SchroederSprite()
{
	
}

void SchroederSprite::Hit(FeeSprite *hitBy)
{	
	if ((hitBy->myKind<GESAMTZAHL_WAFFEN)||(hitBy->myKind==EXPLO_KIND))
	{
		if (hitBy->myKind!=EXPLO_KIND) // wenn es keine Explosion ist
		{
			FeeSprite *temp;
			temp=(dynamic_cast<ShotSprite*>(hitBy))->Owner(); 
			if (temp!=NULL)
				curseAt=temp; // getroffen von wem?
		}
		switch(state)
		{
		case SCHROEDER_PLAYING: 	// wenn wir beim spielen waren,
			{						// mssen wir erst auf die normale Animation
				state=SCHROEDER_CURSING;
				InitFace();
			}//drop through 
		case SCHROEDER_WALKING:
			{
				state=SCHROEDER_CURSING;
				theExplosionSound->Play();
				FlyingPartSprite::SpawnFlyingParts(x, y, myManager->options->GetDataPath("Grafiken/PianoParts.tga"), 20, myManager,14); // Teile losfliegen lassen
			}
		case SCHROEDER_RETURNING:	// beim zurcklaufen
			{
				processCount=0;
				state=SCHROEDER_CURSING; // nur fluchen
			}
		}
	}
}

void SchroederSprite::Update(float time)
{
	if (DoAction(time))
	{
		switch(state)
		{
		case SCHROEDER_SLEEPING:
			{
				if (sge_Random(0,1000)>995) // ab und zu losgehen
				{
					state=SCHROEDER_WALKING;				
					frame=11;
					InitFace();
				}
				break;
			}
		case SCHROEDER_WALKING:
			{
				x++;
				if (((int)x%4)==0)
				{
					frame +=1;
					if (frame>=14) frame=11;
					InitFace();
				}
				if (x>myManager->maxX/2) // in der Mitte des BS spielen
				{
					state=SCHROEDER_PLAYING;
					frame=0;
					InitFace();
				}
				break;
			}
		case SCHROEDER_RETURNING:
			{
				x--;
				if (((int)x%4)==0)
				{
					frame +=1;
					if (frame>=11) frame=8; // zurcklatschen
					InitFace();
				}
				if (x<=-24) state=SCHROEDER_SLEEPING; // bis ans Ende
				break;
			}
		case SCHROEDER_CURSING:
			{
				if (curseAt!=NULL) // fluchen auf Angreifer
				{
					if(curseAt->x>x)
						frame=(processCount/4)%4; // nach rechts gucken
					else							// oder
						frame=4+(processCount/4)%4; // nach links gucken
				}
				else
					frame=(processCount/4)%4; // nach rechts gucken
				InitFace();
				if((processCount++)>50) 
				{
					state=SCHROEDER_RETURNING;		
					processCount=0;
					frame=8;
					InitFace();
				}
				break;
			}
		case SCHROEDER_PLAYING:
			{
				processCount++;
				if ((sge_Random(0,1000)>900)&&(processCount>50))
				{
					new NoteSprite(x,y,sge_Random(0,4),myManager);
					frame=1;
					processCount=0;
				}
				if (processCount>8)
					frame=0;
				InitFace();
			}
		}
	}
}

void SchroederSprite::InitFace()
{
	if (state==SCHROEDER_PLAYING)
	{
		face=myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/Piano.tga"),frame);
		collisionMask=myManager->imageManager->GetFrameMask(myManager->options->GetDataPath("Grafiken/Piano.tga"),frame);
	}
	else
	{
		face=myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/Schroeder.tga"),frame);
		collisionMask=myManager->imageManager->GetFrameMask(myManager->options->GetDataPath("Grafiken/Schroeder.tga"),frame);
	}
}
