// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

#include "SnoopyGlobals.h"
#include "FeeGlobal.h"
#include "ShotSprite.h"
#include "FeeManager.h"
#include "ExplosionSprite.h"
#include "PlayerSprite.h"
#include "FeeImageManager.h"
#include "FeeSound.h"
#include "FeeSprite.h"
#include <math.h>
#include "GameOptions.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// TODO : level 1 munition schwarz

ShotSprite::ShotSprite(int x, int y, int kind, int angle, int speed, PlayerSprite *theOwner, FeeManager *derManager, int isDark):MovingSprite(x,y, derManager)
{
	owner = theOwner;
	this->angle=angle;
	if (kind==ARROW_KIND)
	{
		this->speed=speed+ARROW_SPEED;
		ComputeSpeed();
	}
	else if (kind==MG_KIND)
	{
		this->speed=speed+MG_SPEED+2;
		ComputeSpeed();
	}
	else if (kind==BOMB_KIND)
	{
		this->speed=speed;
		ComputeSpeed();
		this->speedY=max(this->speedY,0);
		this->speedX=0;
		this->angle=4;
        theStartBombSound->Play();
	}
	else if (kind==HM_KIND)
	{
		target=dynamic_cast<PlayerSprite*>(myManager->GetSprite(PLAYER_KIND));
		if (target==owner) target=dynamic_cast<PlayerSprite*>(myManager->GetNextSprite(PLAYER_KIND));

		turnCountdown=0;
		
		this->speed=speed+ARROW_SPEED-2;
		ComputeSpeed();
	}

	myKind=kind;
	age=0;
	this->isDark=isDark;

	if (kind==ARROW_KIND)
		name="Arrow";
	else if (kind==MG_KIND)
		name="MG";
	else if (kind==BOMB_KIND)
		name="Vertically deployed antipersonnel device";
	else if (kind==HM_KIND)
		name="Homing missile";
	canCollide=false; // erst nach dem ersten Update kollidieren
	
}

ShotSprite::~ShotSprite()
{

}

void ShotSprite::Update(float time)
{	
	if (!(face))
		InitFace();

	MovingSprite::Update(time);



	if (myKind==BOMB_KIND)
		if (y>myManager->maxY) toBeRemoved=true; // Vertically deployed antipersonnel devices verschwinden, wenn sie unten 'rausfallen
	
	if (DoAction(time))
	{
		age++;
		if (age==2)
			canCollide=true;
		
		if (myKind==BOMB_KIND)
			if ((speedY++)>FALL_SPEED) speedY=FALL_SPEED;

		if (myKind==HM_KIND)
		{
			turnCountdown++;
			if (turnCountdown>=3)
			{
				int angle1;
				int ydif=y-target->y;
				int xdif=x-target->x;
		
				// bse!
				if (abs(xdif)>(myManager->maxX/2)) xdif*=-1; // heute sind wir mal richtig unsozial :-)
				// bse!
		
				angle1=int(((atan2(ydif,xdif)+PI)*8)/PI)%16; // Winkel ermitteln

				int angle2=angle1;

				if (angle2 < 8)
					angle2 += 16;
				else
					angle2 -= 16;

				if (abs(angle1 - angle) > abs(angle2 - angle)) // wierum drehen?
					angle1 = angle2;

				if (angle1 > angle) // Drehung durchfhren
					angle+=1;
				else
					angle-=1;

				turnCountdown=0;
				ComputeSpeed();
				InitFace();
			}
		}
	}

	if (x > myManager->maxX) 
	{
		x += -29 -myManager->maxX;
		if ((age)>100)
			toBeRemoved=true;
	}
	else if (x < (-29)) 
	{
		x = myManager->maxX +29 + x;
		if ((age)>100)
			toBeRemoved=true;
	}

	if (y > (myManager->maxY)) 
	{
		y += -29 -myManager->maxY; // an der Decke rausgeflogenen Spieler
		if ((age)>100)
			toBeRemoved=true;
	}
	else if (y < (-29))
	{
		y = myManager->maxY +29 + y;    // auf den Boden bringen
		if ((age)>100)
			toBeRemoved=true;
	}
	
	return;
}

void ShotSprite::Hit(FeeSprite *hitBy)
{
	if ((myKind==MG_KIND)||((myKind==ARROW_KIND)&&(true)))
	{
		if ((hitBy->myKind!=MG_KIND)&&(hitBy->myKind!=ARROW_KIND))
			toBeRemoved=true;
	}
	else
		toBeRemoved=true;
	if (myKind==BOMB_KIND)
		new ExplosionSprite(x+10,y+8,myManager);
	return;
}

void ShotSprite::InitFace()
{
	if ((myKind==ARROW_KIND)||(myKind==HM_KIND))
	{
		myManager->imageManager->PrecacheFrames(myManager->options->GetDataPath("Grafiken/Arrow.tga"),16);
		face=myManager->imageManager->GetFrame(myManager->options->GetDataPath("Grafiken/Arrow.tga"),angle);
		collisionMask=myManager->imageManager->GetFrameMask(myManager->options->GetDataPath("Grafiken/Arrow.tga"),angle);
		frame=angle;
	}
	else if (myKind==MG_KIND)
	{
		if (!isDark)
			InitFaceFromIMG(myManager->options->GetDataPath("Grafiken/MG dunkel.tga"));
		else if (isDark==1)
			InitFaceFromIMG(myManager->options->GetDataPath("Grafiken/MG.tga"));
		else 
			InitFaceFromIMG(myManager->options->GetDataPath("Grafiken/MG.tga"));
	}
	else if (myKind==BOMB_KIND)
	{
		InitFaceFromIMG(myManager->options->GetDataPath("Grafiken/Bomb.tga"));
	}
}

PlayerSprite * ShotSprite::Owner()
{
	return owner;
}
