// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

// StatusSprite.cpp: implementation of the StatusSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "FeeSprite.h"
#include "StatusSprite.h"
#include "PlayerSprite.h"
#include "SDL.h"
#include "FeeManager.h"
#include "main.h"
#include "SnoopyGlobals.h"
#include "WoodstockSprite.h"
#include "PumpkinSprite.h"
#include "CloudSprite.h"
#include "BounderSprite.h"
#include "sge_misc.h"
#include "FeeImageManager.h"
#include "ParaSprite.h"
#include "CharlieSprite.h"
#include "SchroederSprite.h"
#include "DecoSprite.h"
#include "SchroederSprite.h"
#include "DT_drawtext.h"
#include "GameOptions.h"

void RenderClouds(SDL_Surface *target);


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

StatusSprite::StatusSprite(PlayerSprite *Spieler1,PlayerSprite *Spieler2, FeeManager *myManager):FeeSprite(0,myManager->maxY,myManager)
{
	face = myManager->imageManager->GetSimpleImage(myManager->options->GetDataPath("Grafiken/Statusbar.tga"));

	numbers = myManager->imageManager->GetImage(myManager->options->GetDataPath("Grafiken/Numbers.tga"));
	matrixmap = myManager->imageManager->GetSimpleImage(myManager->options->GetDataPath("Grafiken/MatrixChars.jpg"));

	myKind=STATUS_KIND;

	canCollide=false;
	renderedClouds=false;
	this->Spieler1=Spieler1;
	this->Spieler2=Spieler2;
	level=1;
}

StatusSprite::~StatusSprite()
{
}

void StatusSprite::Update(float time)
{
	if (Spieler1->kills>19) Spieler1->battles++;
	if (Spieler2->kills>19) Spieler2->battles++;
	if ((Spieler2->kills>19)||Spieler1->kills>19)
	{
		NewLevel();
	}

	if (DoAction(time))
	{
		if ((level == 0) && (!myManager->GetSprite(PUMPKIN_KIND))) // Sind wir im Pumpkinlevel, und ist gerade kein Pumpkin da?
			if (sge_Random(0,1000) > 995) new PumpkinSprite(sge_Random(0, myManager->maxX), myManager->maxY - BODEN - 38, myManager);
		if (level==5) // Matrix
		{
			int character=156;

			matrix_x=sge_Random(0,myManager->maxX/17);
			matrix_y=sge_Random(0,myManager->maxY/21);
			DrawMatrix(matrix_x,matrix_y,character,false); // zwei loeschen

			matrix_x=sge_Random(0,myManager->maxX/17);
			matrix_y=sge_Random(0,myManager->maxY/21);
			DrawMatrix(matrix_x,matrix_y,character,false);

			matrix_x=sge_Random(0,myManager->maxX/17);
			matrix_y=sge_Random(0,myManager->maxY/21);	
			character=sge_Random(0,156);
			DrawMatrix(matrix_x,matrix_y,character,true); // zwei helle

			matrix_x=sge_Random(0,myManager->maxX/17);
			matrix_y=sge_Random(0,myManager->maxY/21);
			character=sge_Random(0,156);
			DrawMatrix(matrix_x,matrix_y,character,true);

			matrix_x=sge_Random(0,myManager->maxX/17);
			matrix_y=sge_Random(0,myManager->maxY/21);
			character=sge_Random(0,156);
			DrawMatrix(matrix_x,matrix_y,character,false);	// und einen dunklen.
		}
		if ((level==1)||(level==6)) // weapons drop
		{
			if (myManager->spriteCount<40)	// keine Kettenreaktion :-(
			{
				int i=sge_Random(0,2000);

				if (i>1990)
					new ParaSprite(sge_Random(0,myManager->maxX),ARROW_KIND,myManager);
				else if(i>1980)
					new ParaSprite(sge_Random(0,myManager->maxX),MG_KIND,myManager);
				else if(i>1976)
					new ParaSprite(sge_Random(0,myManager->maxX),BOMB_KIND,myManager);
				else if((i>1975)&&(myManager->options->withHM==1))
					new ParaSprite(sge_Random(0,myManager->maxX),HM_KIND,myManager);
			}
		}
	}
	switch (level)
	{
	case 1:
	case 6:
		break;
	case 2:
	case 4:
	case 5:
		Spieler1->Reload(ARROW_KIND);
		Spieler2->Reload(ARROW_KIND);
		Spieler1->ChooseWeapon(ARROW_KIND);
		Spieler2->ChooseWeapon(ARROW_KIND);
		break;
	default:
		Spieler1->Reload(MG_KIND);
		Spieler2->Reload(MG_KIND);
		Spieler1->ChooseWeapon(MG_KIND);
		Spieler2->ChooseWeapon(MG_KIND);
		break;
	}
}

void StatusSprite::Hit(FeeSprite *hitBy)
{
}

void StatusSprite::DrawNum(int x,int num, SDL_Surface *target)
{
	SDL_Rect pos1={0,0,face->w,face->h};
	SDL_Rect pos2;

	pos2.w=face->w;
	pos2.h=face->h;
	pos2.x=x+5;
	pos1.x=(num%10)*4;
	pos2.y=y+4;
	pos1.w=4;
	pos1.h=7;
	SDL_BlitSurface(numbers,&pos1,target,&pos2);
	pos2.w=face->w;
	pos2.h=face->h;
	pos2.x=x;
	pos1.x=(num/10)*4;
	pos2.y=y+4;
	pos1.w=4;
	pos1.h=7;
	if (num>9)
		SDL_BlitSurface(numbers,&pos1,target,&pos2);
}

void StatusSprite::DrawAmmoBoard(PlayerSprite* Player,int x, int y, SDL_Surface *target)
{
	char num[4];
	ammoBoard = SDL_DisplayFormat(myManager->imageManager->GetSimpleImage(myManager->options->GetDataPath("Grafiken/Ammoboard.tga")));

	if (myManager->GetLOD()>=1)
		SDL_SetAlpha(ammoBoard,SDL_SRCALPHA,128);

	DT_DrawText(Player->name,ammoBoard,0,3,1);
	sprintf(num, "%i", Player->GetAmmo(ARROW_KIND));
	DT_DrawText(num,ammoBoard,0,18,13);
	sprintf(num, "%i", Player->GetAmmo(MG_KIND));
	DT_DrawText(num,ammoBoard,0,18,25);
	sprintf(num, "%i", Player->GetAmmo(BOMB_KIND));
	DT_DrawText(num,ammoBoard,0,45,20);
	SDL_Rect dest={x,y,0,0};
	SDL_BlitSurface(ammoBoard,NULL,target,&dest);
	SDL_FreeSurface(ammoBoard);
}


void StatusSprite::DrawTo(SDL_Surface *target)
{
	if (face==NULL) return;
	SDL_Rect pos1={0,0,face->w,face->h}; // FIXME - Klappt nicht unter OpenGL
	SDL_Rect pos2;
	
	pos2.x=x;
	pos2.y=y;
	pos2.w=face->w;
	pos2.h=face->h;
	SDL_BlitSurface(face,&pos1,target,&pos2);
	DrawNum(166,Spieler2->kills,target);
	DrawNum(422,Spieler1->kills,target);
	DrawNum(230,Spieler2->battles,target);
	DrawNum(480,Spieler1->battles,target);

	if (Spieler1->ShowAmmo())
	{
		DrawAmmoBoard(Spieler1,15,15,target);
	}
	if (Spieler2->ShowAmmo())
	{
		DrawAmmoBoard(Spieler2,430,15,target);
	}
}


void StatusSprite::ClearLevel()
{
	FeeList *list=myManager->spriteList;
	while (list)
	{
		switch (list->mySprite->myKind)
		{
		case CLOUD_KIND:
		case SKY_KIND:
		case GROUND_KIND:
		case ARROW_KIND:
		case MG_KIND:
		case EXPLO_KIND:
		case SMOKE_KIND:
		case WOODSTOCK_KIND:
		case PUMPKIN_KIND:
		case PARA_KIND:
		case CHARLIE_KIND:
		case BASEBALL_KIND:
		case SCHROEDER_KIND:
		case DECO_KIND:
		case FLYING_PART_KIND:
		case NOTE_KIND:
			list->mySprite->toBeRemoved=true;
			break;
		}
		list=list->next;
	}
}

void StatusSprite::NewLevel()
{
	ClearLevel();
	level++;
	level%=7;
	Spieler1->isDark=1;
	Spieler2->isDark=1;
	switch (level)
	{
	case 0:
		{
			myManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/PumpkinBG.jpg"));
			SDL_Rect sky={0,0,myManager->maxX,3};
			SDL_Rect ground={0,myManager->maxY-BODEN+1,myManager->maxX,15};
			new BounderSprite(sky,SKY_KIND,myManager);
			new BounderSprite(ground,GROUND_KIND,myManager);
			levelName="The Great Pumpkin";
			Spieler1->isDark=2;
			Spieler2->isDark=2;
			break;
		}
	case 1:
		{
			if (myManager->GetLOD()>=4)
			{
				myManager->NewTransparentBkgnd(myManager->options->GetDataPath("Grafiken/SnoopyBG.tga"));
				if (!renderedClouds)
				{
					SDL_Surface *stencil=SDL_DisplayFormatAlpha(myManager->GetBkgnd());
					//SDL_SetColorKey(stencil, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(stencil->format,0xFF,0xFF,0xFF));
					SDL_Surface *target=myManager->GetBkgnd();
					SDL_FillRect(target,NULL,0);
					RenderClouds(target);
					SDL_Rect rect={0,0,stencil->w,stencil->h};
					SDL_BlitSurface(stencil,&rect,target,&rect);
					SDL_FreeSurface(stencil);
					renderedClouds=true;
				}
			}
			else
				myManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/SnoopyBG.jpg"));

			
			SDL_Rect sky={0,0,myManager->maxX,3};
			SDL_Rect ground={0,myManager->maxY-BODEN+1,myManager->maxX,15};
			new BounderSprite(sky,SKY_KIND,myManager);
			new BounderSprite(ground,GROUND_KIND,myManager);			
			levelName="Weapons drop";
			//Spieler1->isDark=0;
			//Spieler2->isDark=0;
			break;
		}
	case 2:
		{
			myManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/WoodstockBG.jpg"));

			new WoodstockSprite(myManager);
			if (myManager->GetLOD()>=1)
				new CloudSprite(myManager);
			if (myManager->GetLOD()>=2)
				new CloudSprite(myManager);
			if (myManager->GetLOD()>=3)
				new CloudSprite(myManager);
			if (myManager->GetLOD()>=4)
			{
				new CloudSprite(myManager);
				new CloudSprite(myManager);
			}
			levelName="Woodstock";
			//Spieler1->isDark=0;
			//Spieler2->isDark=0;
			break;
		}
	case 3:
		{
			myManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/CharlieBrownBG.jpg"));
			SDL_Rect sky={0,0,myManager->maxX,3};
			SDL_Rect ground={0,myManager->maxY-BODEN+1,myManager->maxX,15};
			new BounderSprite(sky,SKY_KIND,myManager);
			new BounderSprite(ground,GROUND_KIND,myManager);
			new CharlieSprite(280,myManager);
			levelName="Charlie Brown";
			break;
		}
	case 4:
		{
			myManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/SchroederBG.jpg"));
			SDL_Rect sky={0,0,myManager->maxX,3};
			SDL_Rect ground={0,myManager->maxY-BODEN+1,myManager->maxX,15};
			new BounderSprite(sky,SKY_KIND,myManager);
			new BounderSprite(ground,GROUND_KIND,myManager);
			if (myManager->GetLOD()>=1)
				new DecoSprite(0,192,myManager,myManager->options->GetDataPath("Grafiken/SchroederHaus.tga"));
			if (myManager->GetLOD()>=3)
				new DecoSprite(0,0,myManager,myManager->options->GetDataPath("Grafiken/SchroederClouds.jpg"),true,true);
			new SchroederSprite(myManager);			
			levelName="Schroeder";
			break;
		}
	case 5:
		{
			myManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/MatrixBG.jpg"));
			SDL_Surface *temp=myManager->GetBkgnd();
			char text[24]="THE MATRIX HAS YOU";
			SDL_Rect rect={0,0,myManager->maxX,myManager->maxY};
			SDL_FillRect(temp,&rect,0);

			if (myManager->GetLOD()==2)
				SDL_SetAlpha(temp,SDL_SRCALPHA|SDL_RLEACCEL,128);
			else if (myManager->GetLOD()>=3)
				SDL_SetAlpha(temp,SDL_SRCALPHA|SDL_RLEACCEL,50);
			else
				SDL_SetAlpha(temp,0,255);

			if (myManager->options->bpp==8)
				SDL_SetAlpha(temp,0,255);
			if (myManager->options->useGL)
			{
				SDL_SetAlpha(temp,0,255); // FIXME - Das wird schwierig. Sehr schwierig.
				strcpy(text,"NO\nMATRIX:IN\rOPENGL!");
			}
			

			for(int i=0;i<22;i++)
			{
				DrawMatrix(i,0,text[i]-65,false);
			}
			levelName="What is the Matrix?";
			
			if (!myManager->options->useGL)
				break;
			else
				level++; // drop through
		}
	case 6:
		{
			myManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/StarsBG.jpg"));
			SDL_Rect sky={0,0,myManager->maxX,3};
			SDL_Rect ground={0,myManager->maxY-BODEN+1,myManager->maxX,15};
			new BounderSprite(sky,SKY_KIND,myManager);
			new BounderSprite(ground,GROUND_KIND,myManager);
			levelName="Weapons drop dark";
			break;
		}
	}
	Spieler1->NewLevel();
	Spieler2->NewLevel();
	/*if (level==5)
	{	
		SDL_SetAlpha(Spieler1->GetFace(),SDL_RLEACCEL,128);
		SDL_SetAlpha(Spieler2->GetFace(),SDL_RLEACCEL,128);				
	}*/
	y=myManager->maxY;
	myManager->MoveSpriteToTop(this);
	return;
}

void StatusSprite::SetLevel(int level)
{
	this->level=level-1;
	NewLevel();
}

int StatusSprite::GetLevel()
{
	return level;
}

void StatusSprite::DrawMatrix(int x, int y, int character, bool light)
{
	SDL_Surface *temp=myManager->GetBkgnd();
	if (!light)
		character+=156;

	SDL_Rect rect={0,0,17,21};
	SDL_Rect rect2={0,0,17,21};
	rect.x=(character%26)*16.5;
	rect.y=((character-(character%26))/26)*21;
	rect2.x=x*16.5;
	rect2.y=y*21;

	SDL_BlitSurface(matrixmap,&rect,temp,&rect2);
}



char *StatusSprite::GetLevelName()
{
	return levelName;
}
