// This program is open source. It is placed under the Q public license.
// Copyright 2000/2001 by J. Fortmann/T.Wiesehfer.

#include "main.h"
#include "FeeManager.h"
#include "PlayerSprite.h"
#include "SnoopyGlobals.h"
#include "PlayerKeys.h"
#include "ShotSprite.h"
#include "StatusSprite.h"
#include "SDL_image.h"
#include "sge_misc.h"
#include "CON_console.h"
#include "PumpkinSprite.h"
#include "FeeImageManager.h"
#include "AIPlayerSprite.h"
#include "HCAIPlayerSprite.h"
#include "GameOptions.h"
#include "DT_drawtext.h"

#ifdef WIN32
#include "winspecific.h"
#endif
#ifdef MACOSX
#include "carbonspecific.h"
#include <Carbon/Carbon.h>
#endif

#include "BounderSprite.h"

#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "SDL.h"


const int PLAYER_COUNT = 2; // 0 und 1

long delayForChange = 0;

FeeManager *theSpriteManager;
StatusSprite *theStatusBar;
GameOptions *options;

bool ConsoleDown=false;
int mspf=1, mspfCon=1, mspfMin=1000, mspfMax=1;

/////////////////////////////////////////////////////////
//
//		Anfang main()
//
/////////////////////////////////////////////////////////

int main( int argc, char* argv[] )
{
	SDL_Surface *theScreen;

#ifdef WIN32
	#ifdef USE_NET
	WSADATA wsadata;					// WinSock will das...
	WSAStartup(MAKEWORD(2,2),&wsadata);
	#endif
#endif
#ifdef MACOSX
        HideMenuBar();
#endif
	sge_Randomize();

	if (SDL_Init(SDL_INIT_TIMER|SDL_INIT_AUDIO|SDL_INIT_VIDEO))
	{
		printf("Could not initialise SDL. (%s)\n",SDL_GetError());
		return 1;
	}

	theSpriteManager =new FeeManager();

	options=theSpriteManager->options;

	options->LoadOptions();

	if (options->useGL)
	{
		theScreen=theSpriteManager->InitializeScreen(1024, 684);
	}
	else
		theScreen=theSpriteManager->InitializeScreen(512, 342 + 15);

	if (!theScreen)
	{
		printf("Could not initialise screen. (%s)\n",SDL_GetError());
		if (options->useGL) 
		{
			printf("You could try to disable OpenGL support.\n");
		}
		return 2;
	}

	SDL_WM_SetCaption("Generic white dog vs. the Red Baron", "Generic white dog vs. the Red Baron");

	theSpriteManager->InitializeSoundSystem();
	InitializeGlobals();
	//theSpriteManager->NewBkgnd(myManager->options->GetDataPath("Grafiken/SnoopyBG.jpg"));

	// Konsole hochfahren
	SDL_Rect consolePos={20,20,450,230};
	if(CON_ConsoleInit(options->GetDataPath("Grafiken/ConsoleFont.tga"), theScreen, 100, consolePos))
		return 1;

	// Konsolen-Kommandos initialisieren
	CON_AddCommand( &PostQuit, "quit");
	CON_AddCommand( &PostQuit, "exit");
	CON_AddCommand( &Move, "move");
	CON_AddCommand( &AlphaChange, "alpha");
	CON_AddCommand( &NextLevel, "level");
	CON_AddCommand( &FramesPerSecond, "fps");
	CON_AddCommand( &PrintScore, "printscore");
	CON_AddCommand( &network, "net");
	CON_AddCommand( &Benchmark, "benchmark");
	CON_AddCommand( &SetLOD, "setLOD");
		
	// auf die Konsole schreiben
	CON_ConOut("Console by Garrett Banuk <mongoose@wpi.edu>");

	CON_ConOut("Snoopy vs. the Red Baron");

	if (sge_Random(0,1000)>500)
	{
		CON_ConOut("Copyright 2000, 2001 by J. Fortmann <PiusII@users.sourceforge.net>");
		CON_ConOut("                        T. Wiesehoefer <YourselfT@users.sourceforge.net>");
	}
	else
	{
		CON_ConOut("Copyright 2000, 2001 by T. Wiesehoefer <YourselfT@users.sourceforge.net>");
		CON_ConOut("                        J. Fortmann <PiusII@users.sourceforge.net>");
	}

	if (theSpriteManager->GetLOD()>=3) CON_ConsoleAlpha(180);
	CON_ListCommands();

	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);

	//PAIPlayerSprite *theRedBaron = new PAIPlayerSprite(100,100,PLAYER_KIND, theSpriteManager,"PAItest.dll");

	//theRedBaron->Activate();

	PlayerSprite *theSnoopy = new PlayerSprite(50,50,PLAYER_KIND, theSpriteManager);
	PlayerSprite *theRedBaron = new HCAIPlayerSprite(100,100,PLAYER_KIND, theSpriteManager);

	theSnoopy->name="Snoopy";
	theRedBaron->name="RedBaron";

	theStatusBar = new StatusSprite(theSnoopy,theRedBaron,theSpriteManager);

	options->Retrieve();

	theSnoopy->myKeys.highspeed=SDLK_w;
	theSnoopy->myKeys.lowspeed=SDLK_s;
	theSnoopy->myKeys.incrangle=SDLK_d;
	theSnoopy->myKeys.decrangle=SDLK_a;
	theSnoopy->myKeys.fire=SDLK_LSHIFT;
	theSnoopy->myKeys.mg=SDLK_e;
	theSnoopy->myKeys.arrow=SDLK_q;
	theSnoopy->myKeys.bomb=SDLK_TAB;

	theSpriteManager->keystateValid=true; // Tasten aktivieren

	sge_CalibrateDelay();

	SDL_Event event;
	int ticks=SDL_GetTicks();
	int oldticks;

	char buf[20];

	// TODO: Konsole in FeeManager einbinden

	while (!(DoEvents(event)))
	{
		
		if(ConsoleDown)
			theSpriteManager->Update((float)mspfCon/50.0f);
		else
			theSpriteManager->Update((float)mspf/50.0f);

		if(ConsoleDown) CON_DrawConsole();

		if ((!options->useGL)&&(options->showFPS))
		{
			sprintf((char*)&buf,"fps: %i",(int)(1000/mspf));
			DT_DrawText((char*)&buf, theScreen, 0, 0, 0 );
		}

		
		theSpriteManager->Flip();

		event.type=0;
		SDL_PollEvent(&event);

		sge_Delay(min(max(50-ticks+oldticks,0),20));

		oldticks = ticks;
		ticks = SDL_GetTicks();
		
		if (!ConsoleDown)
		{
			mspf = ticks-oldticks;
			if (mspfMax-mspf>5) mspfMax=mspf;
			mspfMin = min(mspfMin,mspf);
			mspfMax = max(mspfMax,mspf);			
		}
		else
		{
			mspfCon = ticks-oldticks;
		}

		mspf=min(mspf,100); // Kompensation fr "angehaltene" Zeit in Optionen-Dialog
	}
	options->Store();
	delete theSpriteManager;
	RemoveGlobals();
	CON_Destroy();
	SDL_Quit();
#ifdef MACOSX
        ShowMenuBar();
#endif

#ifdef WIN32
	#ifdef USE_NET
	WSACleanup();			// und weg mit WinSock.
	#endif
#endif
	return 0;
}

/////////////////////////////////////////////////////////
//
//		Ende main()
//
/////////////////////////////////////////////////////////




int DoEvents(SDL_Event event)
{
	Uint8 *keystate = SDL_GetKeyState(NULL);
	switch(event.type)
	{
		/*
#ifdef HAVE_GL_H
	case SDL_VIDEORESIZE:
		{
		SDL_SetVideoMode(event.resize.w, event.resize.h, 0, SDL_OPENGL|SDL_RESIZABLE);
        glClearColor(0, 0, 0, 1);
        glClearDepth(1);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_DITHER);
		glDisable(GL_DEPTH_TEST);
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
        glMatrixMode(GL_PROJECTION);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		glDisable(GL_ALPHA_TEST);
		glDisable(GL_NORMALIZE);
		glShadeModel(GL_SMOOTH);
        glLoadIdentity();
        glMatrixMode(GL_MODELVIEW);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();
		SDL_Rect pos={0,0,512,342};
        glOrtho(pos.x,pos.w,pos.h,pos.y,-1,1);
		break;
	}
#endif*/
	case SDL_KEYDOWN:
		switch(event.key.keysym.sym)
		{
			case SDLK_DELETE:
			default:
				break;
		}
		break;
	case SDL_QUIT:
		return 1;
	default:
		break;
	}



	if (keystate[SDLK_o])
	{
		#if defined WIN32 || defined MACOSX 
		if (!ConsoleDown)
			return ShowOptions();
		#endif
	}

	if (keystate[SDLK_BACKSPACE])
	{
		if (!ConsoleDown) 
		{
			theStatusBar->NewLevel();
			while(keystate[SDLK_BACKSPACE]) 
			{
				SDL_PumpEvents();
				keystate = SDL_GetKeyState(NULL);
			}
		}
	}

	if (keystate[SDLK_RETURN])
	{
		if (!ConsoleDown) 
		{
			options->useAI=!options->useAI;

			options->OptionsChanged();

			while(keystate[SDLK_RETURN]) 
			{
				SDL_PumpEvents();
				keystate = SDL_GetKeyState(NULL);
			}
		}
	}
	
	if (keystate[SDLK_DELETE])
	{
		if(!ConsoleDown)
		{
			ConsoleDown=true;
			theSpriteManager->keystateValid=false;
			SDL_EnableUNICODE(1);
		}
		else
		{
			ConsoleDown=false;
			theSpriteManager->keystateValid=true;
			SDL_EnableUNICODE(0);
		}
		
		while(keystate[SDLK_DELETE]) 
		{
			SDL_PumpEvents();
			keystate = SDL_GetKeyState(NULL);
		}
	}
	if (keystate[SDLK_ESCAPE])
		return 1;

	/* If the console is down send events that were not
	 * filtered out yet to the console */
	if(ConsoleDown)
	{
		CON_ConsoleEvents(&event);
		while (SDL_PollEvent(&event))
		{
			if (event.type==SDL_QUIT)
				return 1;
			CON_ConsoleEvents(&event);
		}
		return 0;
	}

	return 0;
}

void NextLevel(char *dummy)
{
	int level=0;
	int res=0;
	res=sscanf(dummy,"%i",&level);
	
	if (res!=1)
	{
		theStatusBar->NewLevel();
		CON_ConOut("Jumped to next level. (%s)",theStatusBar->GetLevelName());
	}
	else
	{
		theStatusBar->SetLevel(level);
		CON_ConOut("Jumped to level %i. (%s)",level,theStatusBar->GetLevelName());
	}	
}

void FramesPerSecond(char *dummy)
{
	int showFPS;

	if (sscanf(dummy,"%i",&showFPS)==1)
		options->showFPS=showFPS;

	CON_ConOut("%i milliseconds per frame, %i w/ console\n",mspf, mspfCon);
	CON_ConOut("That's %i possible frames per second, resp. %i w/ console", 1000/mspf, 1000/mspfCon);
	CON_ConOut("The minimum time for a frame was %i, the maximum %i", mspfMin, mspfMax);
	mspfMin=1000;
	mspfMax=0;
}

void PrintScore(char *dummy)
{
	PlayerSprite *player=dynamic_cast<PlayerSprite*>(theSpriteManager->GetSprite(PLAYER_KIND));
	CON_ConOut("%s lost %i games, and now has been killed %i times.\n",player->name, player->battles, player->kills);
	player=dynamic_cast<PlayerSprite*>(theSpriteManager->GetNextSprite(PLAYER_KIND));
	CON_ConOut("%s lost %i games, and now has been killed %i times.\n",player->name, player->battles, player->kills);
}

void network(char *string)
{
#if 0
	char hostname[250];
	int port=0;
	char command[20];
	sscanf(string, "%s", command);
		
	if ( (!strcmp(command,"client"))||(!strcmp(command,"server")) )
	{
		if( 2 != sscanf(string, "%s %s %i", command, hostname, &port ))
		{
			strcpy(hostname,"1.0.0.1");
			port=1050;
		}

		if (!strcmp(command,"client"))
			theSpriteManager->Client(hostname,(unsigned short)port);	
		else
			theSpriteManager->Server(hostname,(unsigned short)port);
	}
	if (!strcmp(command,"send"))
	{
		char message[250];
		sscanf(string, "%s %s", command, message);
		theSpriteManager->SendCommand(CMD_TEST,-1,-1,message);
	}
	if (!strcmp(command,"sync"))
	{
		char message[250];
		sscanf(string, "%s %s", command, message);
		theSpriteManager->SendCommand(CMD_FRAMESYNC,-1,-1,message);
	}
	if (!strcmp(command,"frame"))
	{
		char message[250];
		CON_ConOut("Frame: %i",theSpriteManager->frame);
	}

#endif
}




void AlphaChange(char *param)
{
	int alpha=atoi(param);
	if (alpha==0) alpha=255;
	CON_ConsoleAlpha(alpha);
	CON_ConOut("Alpha set to %i.", alpha);
}

void PostQuit(char *dummy)
{
	SDL_Event quit;
	quit.type=SDL_QUIT;
	SDL_PushEvent(&quit);
}


/* Move the console, takes and x and a y */
void Move(char *string)
{
	int	x, y;


	if( 2 != sscanf(string, "%d %d", &x, &y))
	{
		x = 0; y = 0;
	}

	CON_Position(x, y);	
}

void Benchmark(char *dummy)
{
	SDL_Surface *bkgnd=SDL_GetVideoSurface();
	SDL_Surface *sprite=theSpriteManager->imageManager->GetImage(options->GetDataPath("Grafiken/Explo.tga"));
	SDL_Rect dest={0,0,64,64};
	SDL_Rect src={0,0,64,64};
	long time;
	long cycles=200;

	int i,n;
	
	time=SDL_GetTicks();
	for(n=0;n<=cycles;n++)
	{
		for (i=0; i<100;i++)
		{
			dest.x=sge_Random(0,theSpriteManager->maxX);
			dest.y=sge_Random(0,theSpriteManager->maxY);
			SDL_BlitSurface(sprite,&src,bkgnd,&dest);
		}
	}
	CON_ConOut("Drawing 100 sprites took an average of %i ms",(SDL_GetTicks()-time)/n);
}

void SetLOD(char *dummy)
{
	int level=0;
	int res=0;
	res=sscanf(dummy,"%i",&level);
	
	if (res!=1)
	{
		CON_ConOut("Syntax: setLOD <level>");
	}
	else
	{
		level=min(level,4);
		level=max(0,level);
		theSpriteManager->SetLOD(level);
		CON_ConOut("Level of detail: %i",level);
	}	
}

